@@ -37,7 +37,9 @@ typedef struct {
 } PrefData;
 
 #define DEFAULT_ENCODING_SETTING "default-charset"
+// FIXME: Refactor this code to remove the need of those globals
 static WebKitSettings *webkit_settings = NULL;
+static GFileMonitor *user_style_sheet_monitor = NULL;
 
 static void
 user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
@@ -60,9 +62,9 @@ user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
 }
 
 static void
-user_style_seet_read_cb (GFile        *file,
-                         GAsyncResult *result,
-                         gpointer      user_data)
+user_style_sheet_read_cb (GFile        *file,
+                          GAsyncResult *result,
+                          gpointer      user_data)
 {
   GFileInputStream *input_stream;
   GOutputStream *output_stream;
@@ -83,6 +85,21 @@ user_style_seet_read_cb (GFile        *file,
   g_object_unref (output_stream);
 }
 
+static void
+user_style_sheet_file_changed (GFileMonitor      *monitor,
+                               GFile             *file,
+                               GFile             *other_file,
+                               GFileMonitorEvent  event_type,
+                               gpointer           user_data)
+{
+  if (event_type == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT) {
+    webkit_user_content_manager_remove_all_style_sheets (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())));
+
+    g_file_read_async (file, G_PRIORITY_DEFAULT, NULL,
+                       (GAsyncReadyCallback)user_style_sheet_read_cb, NULL);
+  }
+}
+
 static void
 webkit_pref_callback_user_stylesheet (GSettings  *settings,
                                       const char *key,
@@ -92,18 +109,30 @@ webkit_pref_callback_user_stylesheet (GSettings  *settings,
 
   value = g_settings_get_boolean (settings, key);
 
-  if (!value)
+  if (!value) {
+    g_clear_object (&user_style_sheet_monitor);
     webkit_user_content_manager_remove_all_style_sheets (WEBKIT_USER_CONTENT_MANAGER (ephy_embed_shell_get_user_content_manager (ephy_embed_shell_get_default ())));
-  else {
+  } else {
     GFile *file;
+    GError *error = NULL;
     char *filename;
 
-    filename = g_build_filename (ephy_dot_dir (), USER_STYLESHEET_FILENAME, NULL);
+    filename = g_build_filename (ephy_profile_dir (), USER_STYLESHEET_FILENAME, NULL);
     file = g_file_new_for_path (filename);
     g_free (filename);
 
     g_file_read_async (file, G_PRIORITY_DEFAULT, NULL,
-                       (GAsyncReadyCallback)user_style_seet_read_cb, NULL);
+                       (GAsyncReadyCallback)user_style_sheet_read_cb, NULL);
+
+    g_assert (user_style_sheet_monitor == NULL);
+    user_style_sheet_monitor = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, &error);
+    if (user_style_sheet_monitor == NULL) {
+      g_warning ("Could not create a file monitor for %s: %s\n", g_file_get_uri (file), error->message);
+      g_error_free (error);
+    } else {
+      g_signal_connect (user_style_sheet_monitor, "changed", G_CALLBACK (user_style_sheet_file_changed), NULL);
+    }
+
     g_object_unref (file);
   }
 }
@@ -245,7 +274,7 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
     if (!g_strcmp0 (languages[i], "system")) {
       ephy_langs_append_languages (array);
     } else if (languages[i][0] != '\0') {
-      char *str = g_ascii_strdown (languages[i], -1);
+      char *str = g_strdup (languages[i]);
       g_array_append_val (array, str);
     }
   }
@@ -366,6 +395,32 @@ webkit_pref_callback_enable_spell_checking (GSettings  *settings,
   }
 }
 
+static void
+webkit_pref_callback_hardware_acceleration_policy (GSettings  *settings,
+                                                   const char *key,
+                                                   gpointer    data)
+{
+  EphyPrefsWebHardwareAccelerationPolicy value;
+
+  value = g_settings_get_enum (settings, key);
+  switch (value) {
+    case EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY_ALWAYS:
+      webkit_settings_set_hardware_acceleration_policy (webkit_settings,
+                                                        WEBKIT_HARDWARE_ACCELERATION_POLICY_ALWAYS);
+      break;
+    case EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY_NEVER:
+      webkit_settings_set_hardware_acceleration_policy (webkit_settings,
+                                                        WEBKIT_HARDWARE_ACCELERATION_POLICY_NEVER);
+      break;
+    case EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY_ON_DEMAND:
+      /* Fallthrough */
+    default:
+      webkit_settings_set_hardware_acceleration_policy (webkit_settings,
+                                                        WEBKIT_HARDWARE_ACCELERATION_POLICY_ON_DEMAND);
+      break;
+  }
+}
+
 static const PrefData webkit_pref_entries[] =
 {
   /* Epiphany font settings */
@@ -420,6 +475,10 @@ static const PrefData webkit_pref_entries[] =
     EPHY_PREFS_WEB_COOKIES_POLICY,
     "accept-policy",
     webkit_pref_callback_cookie_accept_policy },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY,
+    "hardware-acceleration-policy",
+    webkit_pref_callback_hardware_acceleration_policy },
 };
 
 static gpointer
@@ -433,6 +492,7 @@ ephy_embed_prefs_init (gpointer user_data)
                                                        "enable-dns-prefetching", TRUE,
                                                        "enable-mediasource", TRUE,
                                                        "javascript-can-open-windows-automatically", TRUE,
+                                                       "enable-back-forward-navigation-gestures", TRUE,
                                                        NULL);
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
