@@ -1,23 +1,25 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2; -*- */
-/*  Copyright © 2008 Xan Lopez <xan@gnome.org>
+/*
+ *  Copyright © 2008 Xan Lopez <xan@gnome.org>
  *  Copyright © 2009 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __EPHY_EMBED_PREFS_H__
-#define __EPHY_EMBED_PREFS_H__
+#pragma once
 
 #include "ephy-embed.h"
 
@@ -33,5 +35,3 @@ void ephy_embed_prefs_set_cookie_accept_policy          (WebKitCookieManager *co
                                                          const char          *settings_policy);
 
 G_END_DECLS
-
-#endif /* __EPHY_EMBED_PREFS_H__ */
