@@ -21,12 +21,14 @@
 
 #pragma once
 
+#include <dazzle.h>
 #include <webkit2/webkit2.h>
 
 #include "ephy-downloads-manager.h"
 #include "ephy-encodings.h"
 #include "ephy-gsb-service.h"
 #include "ephy-history-service.h"
+#include "ephy-password-manager.h"
 #include "ephy-permissions-manager.h"
 #include "ephy-search-engine-manager.h"
 
@@ -34,7 +36,7 @@ G_BEGIN_DECLS
 
 #define EPHY_TYPE_EMBED_SHELL (ephy_embed_shell_get_type ())
 
-G_DECLARE_DERIVABLE_TYPE (EphyEmbedShell, ephy_embed_shell, EPHY, EMBED_SHELL, GtkApplication)
+G_DECLARE_DERIVABLE_TYPE (EphyEmbedShell, ephy_embed_shell, EPHY, EMBED_SHELL, DzlApplication)
 
 typedef enum
 {
@@ -44,17 +46,19 @@ typedef enum
   EPHY_EMBED_SHELL_MODE_INCOGNITO,
   EPHY_EMBED_SHELL_MODE_APPLICATION,
   EPHY_EMBED_SHELL_MODE_TEST,
-  EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER
+  EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER,
+  EPHY_EMBED_SHELL_MODE_AUTOMATION
 } EphyEmbedShellMode;
 
 struct _EphyEmbedShellClass
 {
-  GtkApplicationClass parent_class;
+  DzlApplicationClass parent_class;
 
   void    (* restored_window)  (EphyEmbedShell *shell);
 };
 
 EphyEmbedShell    *ephy_embed_shell_get_default                (void);
+const char        *ephy_embed_shell_get_guid                   (EphyEmbedShell   *shell);
 WebKitWebContext  *ephy_embed_shell_get_web_context            (EphyEmbedShell   *shell);
 EphyHistoryService
                   *ephy_embed_shell_get_global_history_service (EphyEmbedShell   *shell);
@@ -68,14 +72,6 @@ void               ephy_embed_shell_set_print_settings         (EphyEmbedShell
                                                                 GtkPrintSettings *settings);
 GtkPrintSettings  *ephy_embed_shell_get_print_settings         (EphyEmbedShell   *shell);
 EphyEmbedShellMode ephy_embed_shell_get_mode                   (EphyEmbedShell   *shell);
-gboolean           ephy_embed_shell_launch_handler             (EphyEmbedShell   *shell,
-                                                                GFile            *file,
-                                                                const char       *mime_type,
-                                                                guint32           user_time);
-void               ephy_embed_shell_add_app_related_uri        (EphyEmbedShell   *shell,
-                                                                const char       *uri);
-gboolean           ephy_embed_shell_uri_looks_related_to_app   (EphyEmbedShell   *shell,
-                                                                const char       *uri);
 void               ephy_embed_shell_clear_cache                (EphyEmbedShell   *shell);
 void               ephy_embed_shell_set_thumbnail_path         (EphyEmbedShell   *shell,
                                                                 const char       *url,
@@ -86,5 +82,6 @@ WebKitUserContentManager *ephy_embed_shell_get_user_content_manager (EphyEmbedSh
 EphyDownloadsManager     *ephy_embed_shell_get_downloads_manager    (EphyEmbedShell *shell);
 EphyPermissionsManager   *ephy_embed_shell_get_permissions_manager  (EphyEmbedShell *shell);
 EphySearchEngineManager  *ephy_embed_shell_get_search_engine_manager (EphyEmbedShell *shell);
+EphyPasswordManager      *ephy_embed_shell_get_password_manager      (EphyEmbedShell *shell);
 
 G_END_DECLS
