@@ -1,5 +1,4 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2; -*- */
-/* vim: set sw=2 ts=2 sts=2 et: */
 /*
  *  Copyright © 2007 Xan Lopez
  *  Copyright © 2008 Jan Alonzo
@@ -7,18 +6,20 @@
  *  Copyright © 2009 Igalia S.L.
  *  Copyright © 2009 Collabora Ltd.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -29,6 +30,7 @@
 #include "ephy-embed-shell.h"
 #include "ephy-embed-utils.h"
 #include "ephy-find-toolbar.h"
+#include "ephy-notification-container.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-string.h"
@@ -44,7 +46,6 @@ static void     ephy_embed_restored_window_cb (EphyEmbedShell *shell,
 
 #define EPHY_EMBED_STATUSBAR_TAB_MESSAGE_CONTEXT_DESCRIPTION "tab_message"
 #define MAX_TITLE_LENGTH 512 /* characters */
-#define EMPTY_PAGE_TITLE _("Blank page") /* Title for the empty page */
 
 typedef struct {
   gchar *text;
@@ -60,6 +61,7 @@ struct _EphyEmbed {
   GtkPaned *paned;
   WebKitWebView *web_view;
   GSList *destroy_on_transition_list;
+  GtkWidget *overlay;
   GtkWidget *floating_bar;
   GtkWidget *progress;
   GtkWidget *fullscreen_message_label;
@@ -85,6 +87,7 @@ struct _EphyEmbed {
   gulong status_handler_id;
   gulong progress_update_handler_id;
   gboolean inspector_loaded;
+  gboolean progress_bar_enabled;
 };
 
 G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_BOX)
@@ -93,6 +96,7 @@ enum {
   PROP_0,
   PROP_WEB_VIEW,
   PROP_TITLE,
+  PROP_PROGRESS_BAR_ENABLED,
   LAST_PROP
 };
 
@@ -133,8 +137,8 @@ ephy_embed_statusbar_get_context_id (EphyEmbed *embed, const char  *context_desc
   char *string;
   guint id;
 
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), 0);
-  g_return_val_if_fail (context_description != NULL, 0);
+  g_assert (EPHY_IS_EMBED (embed));
+  g_assert (context_description != NULL);
 
   /* we need to preserve namespaces on object datas */
   string = g_strconcat ("ephy-embed-status-bar-context:", context_description, NULL);
@@ -165,7 +169,7 @@ ephy_embed_set_statusbar_label (EphyEmbed *embed, const char *label)
 static void
 ephy_embed_statusbar_update (EphyEmbed *embed, const char *text)
 {
-  g_return_if_fail (EPHY_IS_EMBED (embed));
+  g_assert (EPHY_IS_EMBED (embed));
 
   ephy_embed_set_statusbar_label (embed, text);
 }
@@ -175,11 +179,11 @@ ephy_embed_statusbar_push (EphyEmbed *embed, guint context_id, const char *text)
 {
   EphyEmbedStatusbarMsg *msg;
 
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), 0);
-  g_return_val_if_fail (context_id != 0, 0);
-  g_return_val_if_fail (text != NULL, 0);
+  g_assert (EPHY_IS_EMBED (embed));
+  g_assert (context_id != 0);
+  g_assert (text != NULL);
 
-  msg = g_slice_new (EphyEmbedStatusbarMsg);
+  msg = g_new (EphyEmbedStatusbarMsg, 1);
   msg->text = g_strdup (text);
   msg->context_id = context_id;
   msg->message_id = embed->seq_message_id++;
@@ -199,8 +203,8 @@ ephy_embed_statusbar_pop (EphyEmbed *embed, guint context_id)
   EphyEmbedStatusbarMsg *msg;
   GSList *list;
 
-  g_return_if_fail (EPHY_IS_EMBED (embed));
-  g_return_if_fail (context_id != 0);
+  g_assert (EPHY_IS_EMBED (embed));
+  g_assert (context_id != 0);
 
   for (list = embed->messages; list; list = list->next) {
     msg = list->data;
@@ -208,7 +212,7 @@ ephy_embed_statusbar_pop (EphyEmbed *embed, guint context_id)
     if (msg->context_id == context_id) {
       embed->messages = g_slist_remove_link (embed->messages, list);
       g_free (msg->text);
-      g_slice_free (EphyEmbedStatusbarMsg, msg);
+      g_free (msg);
       g_slist_free_1 (list);
       break;
     }
@@ -260,7 +264,7 @@ ephy_embed_set_title (EphyEmbed  *embed,
 
     if (new_title == NULL || new_title[0] == '\0') {
       g_free (new_title);
-      new_title = g_strdup (EMPTY_PAGE_TITLE);
+      new_title = g_strdup (_(BLANK_PAGE_TITLE));
     }
   }
 
@@ -410,7 +414,7 @@ ephy_embed_finalize (GObject *object)
 
     msg = list->data;
     g_free (msg->text);
-    g_slice_free (EphyEmbedStatusbarMsg, msg);
+    g_free (msg);
   }
 
   g_slist_free (embed->messages);
@@ -442,6 +446,9 @@ ephy_embed_set_property (GObject      *object,
     case PROP_TITLE:
       ephy_embed_set_title (embed, g_value_get_string (value));
       break;
+    case PROP_PROGRESS_BAR_ENABLED:
+      embed->progress_bar_enabled = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -463,6 +470,9 @@ ephy_embed_get_property (GObject    *object,
     case PROP_TITLE:
       g_value_set_string (value, ephy_embed_get_title (embed));
       break;
+    case PROP_PROGRESS_BAR_ENABLED:
+      g_value_set_boolean (value, embed->progress_bar_enabled);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -505,6 +515,13 @@ ephy_embed_class_init (EphyEmbedClass *klass)
                          NULL,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
+  obj_properties[PROP_PROGRESS_BAR_ENABLED] =
+    g_param_spec_boolean ("progress-bar-enabled",
+                          "Progress bar",
+                          "Whether to show progress bar within embed",
+                          TRUE,
+                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
   g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 }
 
@@ -702,7 +719,6 @@ ephy_embed_constructed (GObject *object)
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   GtkWidget *paned;
   WebKitWebInspector *inspector;
-  GtkWidget *overlay;
 
   g_signal_connect (shell, "window-restored",
                     G_CALLBACK (ephy_embed_restored_window_cb), embed);
@@ -711,12 +727,12 @@ ephy_embed_constructed (GObject *object)
                     G_CALLBACK (ephy_embed_mapped_cb), NULL);
 
   /* Skeleton */
-  overlay = gtk_overlay_new ();
+  embed->overlay = gtk_overlay_new ();
 
-  gtk_widget_add_events (overlay,
+  gtk_widget_add_events (embed->overlay,
                          GDK_ENTER_NOTIFY_MASK |
                          GDK_LEAVE_NOTIFY_MASK);
-  gtk_container_add (GTK_CONTAINER (overlay), GTK_WIDGET (embed->web_view));
+  gtk_container_add (GTK_CONTAINER (embed->overlay), GTK_WIDGET (embed->web_view));
 
   /* Floating message popup for fullscreen mode. */
   embed->fullscreen_message_label = gtk_label_new (NULL);
@@ -724,7 +740,7 @@ ephy_embed_constructed (GObject *object)
   gtk_widget_set_halign (embed->fullscreen_message_label, GTK_ALIGN_CENTER);
   gtk_widget_set_valign (embed->fullscreen_message_label, GTK_ALIGN_CENTER);
   gtk_widget_set_no_show_all (embed->fullscreen_message_label, TRUE);
-  gtk_overlay_add_overlay (GTK_OVERLAY (overlay), embed->fullscreen_message_label);
+  gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), embed->fullscreen_message_label);
   ephy_embed_set_fullscreen_message (embed, FALSE);
 
   /* statusbar is hidden by default */
@@ -733,14 +749,17 @@ ephy_embed_constructed (GObject *object)
   gtk_widget_set_valign (embed->floating_bar, GTK_ALIGN_END);
   gtk_widget_set_no_show_all (embed->floating_bar, TRUE);
 
-  gtk_overlay_add_overlay (GTK_OVERLAY (overlay), embed->floating_bar);
+  gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), embed->floating_bar);
+  gtk_overlay_set_overlay_pass_through (GTK_OVERLAY (embed->overlay), embed->floating_bar, TRUE);
 
-  embed->progress = gtk_progress_bar_new ();
-  gtk_style_context_add_class (gtk_widget_get_style_context (embed->progress),
-                               GTK_STYLE_CLASS_OSD);
-  gtk_widget_set_halign (embed->progress, GTK_ALIGN_FILL);
-  gtk_widget_set_valign (embed->progress, GTK_ALIGN_START);
-  gtk_overlay_add_overlay (GTK_OVERLAY (overlay), embed->progress);
+  if (embed->progress_bar_enabled) {
+    embed->progress = gtk_progress_bar_new ();
+    gtk_style_context_add_class (gtk_widget_get_style_context (embed->progress),
+                                 GTK_STYLE_CLASS_OSD);
+    gtk_widget_set_halign (embed->progress, GTK_ALIGN_FILL);
+    gtk_widget_set_valign (embed->progress, GTK_ALIGN_START);
+    gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), embed->progress);
+  }
 
   embed->find_toolbar = ephy_find_toolbar_new (embed->web_view);
   g_signal_connect (embed->find_toolbar, "close",
@@ -753,9 +772,11 @@ ephy_embed_constructed (GObject *object)
 
   paned = GTK_WIDGET (embed->paned);
 
-  embed->progress_update_handler_id = g_signal_connect (embed->web_view, "notify::estimated-load-progress",
-                                                        G_CALLBACK (progress_update), object);
-  gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (overlay),
+  if (embed->progress_bar_enabled)
+    embed->progress_update_handler_id = g_signal_connect (embed->web_view, "notify::estimated-load-progress",
+                                                          G_CALLBACK (progress_update), object);
+
+  gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (embed->overlay),
                    TRUE, FALSE);
 
   gtk_box_pack_start (GTK_BOX (embed),
@@ -787,6 +808,21 @@ ephy_embed_constructed (GObject *object)
   g_signal_connect (inspector, "closed",
                     G_CALLBACK (ephy_embed_close_inspector_cb),
                     embed);
+
+  if (webkit_web_view_is_controlled_by_automation (embed->web_view)) {
+    GtkWidget *info_bar;
+    GtkWidget *label;
+
+    info_bar = gtk_info_bar_new ();
+    gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_INFO);
+    /* Translators: this means WebDriver control. */
+    label = gtk_label_new (_("Web is being controlled by automation."));
+    gtk_box_pack_start (GTK_BOX (gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar))), label, FALSE, FALSE, 0);
+    gtk_widget_show (label);
+
+    ephy_embed_add_top_widget (embed, info_bar, EPHY_EMBED_TOP_WIDGET_POLICY_RETAIN_ON_TRANSITION);
+    gtk_widget_show (info_bar);
+  }
 }
 
 static void
@@ -814,7 +850,7 @@ ephy_embed_init (EphyEmbed *embed)
 EphyWebView *
 ephy_embed_get_web_view (EphyEmbed *embed)
 {
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
+  g_assert (EPHY_IS_EMBED (embed));
 
   return EPHY_WEB_VIEW (embed->web_view);
 }
@@ -830,7 +866,7 @@ ephy_embed_get_web_view (EphyEmbed *embed)
 EphyFindToolbar *
 ephy_embed_get_find_toolbar (EphyEmbed *embed)
 {
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
+  g_assert (EPHY_IS_EMBED (embed));
 
   return EPHY_FIND_TOOLBAR (embed->find_toolbar);
 }
@@ -840,17 +876,19 @@ ephy_embed_get_find_toolbar (EphyEmbed *embed)
  * ephy_embed_add_top_widget:
  * @embed: an #EphyEmbed
  * @widget: a #GtkWidget
- * @destroy_on_transition: whether the widget be automatically
+ * @policy: whether the widget be automatically
  * destroyed on page transitions
  *
  * Adds a #GtkWidget to the top of the embed.
  */
 void
-ephy_embed_add_top_widget (EphyEmbed *embed, GtkWidget *widget, gboolean destroy_on_transition)
+ephy_embed_add_top_widget (EphyEmbed                *embed,
+                           GtkWidget                *widget,
+                           EphyEmbedTopWidgetPolicy  policy)
 {
   GSList *list;
 
-  if (destroy_on_transition) {
+  if (policy == EPHY_EMBED_TOP_WIDGET_POLICY_DESTROY_ON_TRANSITION) {
     list = embed->destroy_on_transition_list;
     list = g_slist_prepend (list, widget);
     embed->destroy_on_transition_list = list;
@@ -900,8 +938,8 @@ ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget)
 void
 ephy_embed_set_delayed_load_request (EphyEmbed *embed, WebKitURIRequest *request, WebKitWebViewSessionState *state)
 {
-  g_return_if_fail (EPHY_IS_EMBED (embed));
-  g_return_if_fail (WEBKIT_IS_URI_REQUEST (request));
+  g_assert (EPHY_IS_EMBED (embed));
+  g_assert (WEBKIT_IS_URI_REQUEST (request));
 
   g_clear_pointer (&embed->delayed_state, webkit_web_view_session_state_unref);
   g_clear_object (&embed->delayed_request);
@@ -922,7 +960,7 @@ ephy_embed_set_delayed_load_request (EphyEmbed *embed, WebKitURIRequest *request
 gboolean
 ephy_embed_has_load_pending (EphyEmbed *embed)
 {
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
+  g_assert (EPHY_IS_EMBED (embed));
 
   return !!embed->delayed_request;
 }
@@ -930,7 +968,7 @@ ephy_embed_has_load_pending (EphyEmbed *embed)
 const char *
 ephy_embed_get_title (EphyEmbed *embed)
 {
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
+  g_assert (EPHY_IS_EMBED (embed));
 
   return embed->title;
 }
@@ -947,7 +985,36 @@ ephy_embed_get_title (EphyEmbed *embed)
 gboolean
 ephy_embed_inspector_is_loaded (EphyEmbed *embed)
 {
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
+  g_assert (EPHY_IS_EMBED (embed));
 
   return embed->inspector_loaded;
 }
+
+void
+ephy_embed_attach_notification_container (EphyEmbed *embed)
+{
+  EphyNotificationContainer *container;
+
+  g_assert (EPHY_IS_EMBED (embed));
+
+  container = ephy_notification_container_get_default ();
+  if (gtk_widget_get_parent (GTK_WIDGET (container)) == NULL)
+    gtk_overlay_add_overlay (GTK_OVERLAY (embed->overlay), GTK_WIDGET (container));
+}
+
+void
+ephy_embed_detach_notification_container (EphyEmbed *embed)
+{
+  EphyNotificationContainer *container;
+
+  g_assert (EPHY_IS_EMBED (embed));
+
+  container = ephy_notification_container_get_default ();
+  if (gtk_widget_get_parent (GTK_WIDGET (container)) == embed->overlay) {
+    /* Since the overlay container will own the one and only reference to the
+     * notification widget, removing it from the container will destroy the
+     * singleton. To prevent this, add a reference to it before removing it
+     * from the container. */
+    gtk_container_remove (GTK_CONTAINER (embed->overlay), g_object_ref (GTK_WIDGET (container)));
+  }
+}
