@@ -188,211 +188,6 @@ ephy_web_extension_proxy_new (GDBusConnection *connection)
   return web_extension;
 }
 
-void
-ephy_web_extension_proxy_form_auth_data_save_confirmation_response (EphyWebExtensionProxy *web_extension,
-                                                                    guint                  request_id,
-                                                                    gboolean               response)
-{
-  g_assert (EPHY_IS_WEB_EXTENSION_PROXY (web_extension));
-
-  if (!web_extension->proxy)
-    return;
-
-  g_dbus_proxy_call (web_extension->proxy,
-                     "FormAuthDataSaveConfirmationResponse",
-                     g_variant_new ("(ub)", request_id, response),
-                     G_DBUS_CALL_FLAGS_NONE,
-                     -1,
-                     web_extension->cancellable,
-                     NULL, NULL);
-}
-
-static void
-has_modified_forms_cb (GDBusProxy   *proxy,
-                       GAsyncResult *result,
-                       GTask        *task)
-{
-  GVariant *return_value;
-  gboolean retval = FALSE;
-
-  return_value = g_dbus_proxy_call_finish (proxy, result, NULL);
-  if (return_value) {
-    g_variant_get (return_value, "(b)", &retval);
-    g_variant_unref (return_value);
-  }
-
-  g_task_return_boolean (task, retval);
-  g_object_unref (task);
-}
-
-void
-ephy_web_extension_proxy_web_page_has_modified_forms (EphyWebExtensionProxy *web_extension,
-                                                      guint64                page_id,
-                                                      GCancellable          *cancellable,
-                                                      GAsyncReadyCallback    callback,
-                                                      gpointer               user_data)
-{
-  GTask *task;
-
-  g_assert (EPHY_IS_WEB_EXTENSION_PROXY (web_extension));
-
-  task = g_task_new (web_extension, cancellable, callback, user_data);
-
-  if (web_extension->proxy) {
-    g_dbus_proxy_call (web_extension->proxy,
-                       "HasModifiedForms",
-                       g_variant_new ("(t)", page_id),
-                       G_DBUS_CALL_FLAGS_NONE,
-                       -1,
-                       web_extension->cancellable,
-                       (GAsyncReadyCallback)has_modified_forms_cb,
-                       g_object_ref (task));
-  } else {
-    g_task_return_boolean (task, FALSE);
-  }
-
-  g_object_unref (task);
-}
-
-gboolean
-ephy_web_extension_proxy_web_page_has_modified_forms_finish (EphyWebExtensionProxy *web_extension,
-                                                             GAsyncResult          *result,
-                                                             GError               **error)
-{
-  g_assert (g_task_is_valid (result, web_extension));
-
-  return g_task_propagate_boolean (G_TASK (result), error);
-}
-
-static void
-get_best_web_app_icon_cb (GDBusProxy   *proxy,
-                          GAsyncResult *result,
-                          GTask        *task)
-{
-  GVariant *retval;
-  GError *error = NULL;
-
-  retval = g_dbus_proxy_call_finish (proxy, result, &error);
-  if (!retval) {
-    g_task_return_error (task, error);
-  } else {
-    g_task_return_pointer (task, retval, (GDestroyNotify)g_variant_unref);
-  }
-  g_object_unref (task);
-}
-
-void
-ephy_web_extension_proxy_get_best_web_app_icon (EphyWebExtensionProxy *web_extension,
-                                                guint64                page_id,
-                                                const char            *base_uri,
-                                                GCancellable          *cancellable,
-                                                GAsyncReadyCallback    callback,
-                                                gpointer               user_data)
-{
-  GTask *task;
-
-  g_assert (EPHY_IS_WEB_EXTENSION_PROXY (web_extension));
-
-  task = g_task_new (web_extension, cancellable, callback, user_data);
-
-  if (web_extension->proxy) {
-    g_dbus_proxy_call (web_extension->proxy,
-                       "GetBestWebAppIcon",
-                       g_variant_new ("(ts)", page_id, base_uri),
-                       G_DBUS_CALL_FLAGS_NONE,
-                       -1,
-                       web_extension->cancellable,
-                       (GAsyncReadyCallback)get_best_web_app_icon_cb,
-                       g_object_ref (task));
-  } else {
-    g_task_return_boolean (task, FALSE);
-  }
-
-  g_object_unref (task);
-}
-
-gboolean
-ephy_web_extension_proxy_get_best_web_app_icon_finish (EphyWebExtensionProxy *web_extension,
-                                                       GAsyncResult          *result,
-                                                       char                 **icon_uri,
-                                                       char                 **icon_color,
-                                                       GError               **error)
-{
-  GVariant *variant;
-  GTask *task = G_TASK (result);
-
-  g_assert (g_task_is_valid (result, web_extension));
-
-  variant = g_task_propagate_pointer (task, error);
-  if (!variant)
-    return FALSE;
-
-  g_variant_get (variant, "(ss)", icon_uri, icon_color);
-  g_variant_unref (variant);
-
-  return TRUE;
-}
-
-static void
-get_web_app_title_cb (GDBusProxy   *proxy,
-                      GAsyncResult *result,
-                      GTask        *task)
-{
-  GVariant *retval;
-  GError *error = NULL;
-
-  retval = g_dbus_proxy_call_finish (proxy, result, &error);
-  if (!retval) {
-    g_task_return_error (task, error);
-  } else {
-    char *title;
-
-    g_variant_get (retval, "(s)", &title);
-    g_task_return_pointer (task, title, (GDestroyNotify)g_free);
-    g_variant_unref (retval);
-  }
-  g_object_unref (task);
-}
-
-void
-ephy_web_extension_proxy_get_web_app_title (EphyWebExtensionProxy *web_extension,
-                                            guint64                page_id,
-                                            GCancellable          *cancellable,
-                                            GAsyncReadyCallback    callback,
-                                            gpointer               user_data)
-{
-  GTask *task;
-
-  g_assert (EPHY_IS_WEB_EXTENSION_PROXY (web_extension));
-
-  task = g_task_new (web_extension, cancellable, callback, user_data);
-
-  if (web_extension->proxy) {
-    g_dbus_proxy_call (web_extension->proxy,
-                       "GetWebAppTitle",
-                       g_variant_new ("(t)", page_id),
-                       G_DBUS_CALL_FLAGS_NONE,
-                       -1,
-                       web_extension->cancellable,
-                       (GAsyncReadyCallback)get_web_app_title_cb,
-                       g_object_ref (task));
-  } else {
-    g_task_return_pointer (task, NULL, NULL);
-  }
-
-  g_object_unref (task);
-}
-
-char *
-ephy_web_extension_proxy_get_web_app_title_finish (EphyWebExtensionProxy *web_extension,
-                                                   GAsyncResult          *result,
-                                                   GError               **error)
-{
-  g_assert (g_task_is_valid (result, web_extension));
-
-  return g_task_propagate_pointer (G_TASK (result), error);
-}
-
 void
 ephy_web_extension_proxy_history_set_urls (EphyWebExtensionProxy *web_extension,
                                            GList                 *urls)
@@ -499,3 +294,45 @@ ephy_web_extension_proxy_history_clear (EphyWebExtensionProxy *web_extension)
                      web_extension->cancellable,
                      NULL, NULL);
 }
+
+void
+ephy_web_extension_proxy_password_cached_users_response (EphyWebExtensionProxy *web_extension,
+                                                         GList                 *users,
+                                                         gint32                 promise_id,
+                                                         guint64                page_id)
+{
+  if (!web_extension->proxy)
+    return;
+
+  GList *l;
+  g_auto(GVariantBuilder) builder = G_VARIANT_BUILDER_INIT (G_VARIANT_TYPE_STRING_ARRAY);
+  for (l = users; l != NULL; l = l->next)
+    g_variant_builder_add (&builder, "s", l->data);
+
+  g_dbus_proxy_call (web_extension->proxy,
+                     "PasswordQueryUsernamesResponse",
+                     g_variant_new ("(asit)", &builder, promise_id, page_id),
+                     G_DBUS_CALL_FLAGS_NONE,
+                     -1,
+                     web_extension->cancellable,
+                     NULL, NULL);
+}
+
+void
+ephy_web_extension_proxy_password_query_response (EphyWebExtensionProxy *web_extension,
+                                                  const char            *username,
+                                                  const char            *password,
+                                                  gint32                 promise_id,
+                                                  guint64                page_id)
+{
+  if (!web_extension->proxy)
+    return;
+
+  g_dbus_proxy_call (web_extension->proxy,
+                     "PasswordQueryResponse",
+                     g_variant_new ("(ssit)", username ?: "", password ?: "", promise_id, page_id),
+                     G_DBUS_CALL_FLAGS_NONE,
+                     -1,
+                     web_extension->cancellable,
+                     NULL, NULL);
+}
\ No newline at end of file
