@@ -26,6 +26,7 @@
 #include "ephy-embed-shell.h"
 #include "ephy-history-types.h"
 #include "ephy-security-levels.h"
+#include "ephy-web-extension-proxy.h"
 
 G_BEGIN_DECLS
 
@@ -40,7 +41,8 @@ G_DECLARE_FINAL_TYPE (EphyWebView, ephy_web_view, EPHY, WEB_VIEW, WebKitWebView)
                                         "^https?://[^/\\.[:space:]]+.*$|" \
                                         "^about:.*$|" \
                                         "^data:.*$|" \
-                                        "^file:.*$" \
+                                        "^file:.*$|" \
+                                        "^inspector://.*$" \
                                         ")"
 
 #define EPHY_WEB_VIEW_DOMAIN_REGEX "^localhost(\\.[^[:space:]]+)?(:\\d+)?(:[0-9]+)?(/.*)?$|" \
@@ -69,6 +71,9 @@ typedef enum {
   EPHY_WEB_VIEW_ERROR_UNSAFE_BROWSING
 } EphyWebViewErrorPage;
 
+typedef void (* EphyPasswordSaveRequestCallback) (int      response_id,
+                                                  gpointer data);
+
 GType                      ephy_web_view_chrome_get_type          (void);
 GType                      ephy_web_view_security_level_get_type  (void);
 GtkWidget *                ephy_web_view_new                      (void);
@@ -110,7 +115,6 @@ void                       ephy_web_view_get_security_level       (EphyWebView
                                                                    GTlsCertificateFlags      *errors);
 void                       ephy_web_view_print                    (EphyWebView               *view);
 const char *               ephy_web_view_get_address              (EphyWebView               *view);
-const char *             ephy_web_view_get_last_committed_address (EphyWebView               *view);
 const char *               ephy_web_view_get_display_address      (EphyWebView               *view);
 void                       ephy_web_view_set_placeholder          (EphyWebView               *view,
                                                                    const char                *uri,
@@ -151,4 +155,20 @@ char *                     ephy_web_view_create_web_application   (EphyWebView
                                                                    const char                *title,
                                                                    GdkPixbuf                 *icon);
 
+void                       ephy_web_view_toggle_reader_mode       (EphyWebView               *view,
+                                                                   gboolean                   active);
+
+gboolean                   ephy_web_view_is_reader_mode_available (EphyWebView               *view);
+
+gboolean                   ephy_web_view_get_reader_mode_state    (EphyWebView               *view);
+
+EphyWebExtensionProxy     *ephy_web_view_get_web_extension_proxy  (EphyWebView               *view);
+
+void                       ephy_web_view_show_auth_form_save_request (EphyWebView                    *web_view,
+                                                                      const char                     *origin,
+                                                                      const char                     *username,
+                                                                      EphyPasswordSaveRequestCallback response_callback,
+                                                                      gpointer                        response_data,
+                                                                      GDestroyNotify                  response_destroy);
+
 G_END_DECLS
