@@ -122,8 +122,7 @@ validate_thumbnail_path (const char *path,
 static char *
 thumbnail_directory (void)
 {
-  return g_build_filename (g_get_user_cache_dir (),
-                           "epiphany",
+  return g_build_filename (ephy_cache_dir (),
                            "thumbnails",
                            NULL);
 }
@@ -203,7 +202,10 @@ save_thumbnail (GdkPixbuf  *pixbuf,
     g_warning ("Failed to create thumbnail %s: %s", tmp_path, error->message);
     g_error_free (error);
   }
-  unlink (tmp_path);
+
+  if (tmp_path != NULL)
+    unlink (tmp_path);
+
   g_free (path);
   g_free (tmp_path);
   g_free (dirname);
@@ -288,7 +290,7 @@ snapshot_async_data_new (EphySnapshotService *service,
 {
   SnapshotAsyncData *data;
 
-  data = g_slice_new0 (SnapshotAsyncData);
+  data = g_new0 (SnapshotAsyncData, 1);
   data->service = g_object_ref (service);
   data->snapshot = snapshot ? g_object_ref (snapshot) : NULL;
   data->web_view = web_view;
@@ -320,8 +322,7 @@ snapshot_async_data_free (SnapshotAsyncData *data)
     g_object_remove_weak_pointer (G_OBJECT (data->web_view), (gpointer *)&data->web_view);
 
   g_free (data->url);
-
-  g_slice_free (SnapshotAsyncData, data);
+  g_free (data);
 }
 
 typedef struct {
