@@ -23,6 +23,7 @@
 
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
+#include "ephy-profile-utils.h"
 #include "ephy-settings.h"
 
 #include <errno.h>
@@ -156,6 +157,19 @@ get_app_id_from_profile_directory (const char *profile_dir)
   return program_name ? get_app_id_from_program_name (program_name) : NULL;
 }
 
+static char *
+ephy_web_application_get_directory_under (const char *id,
+                                          const char *path)
+{
+  g_autofree char *app_dir = NULL;
+
+  app_dir = get_app_profile_directory_name (id);
+  if (!app_dir)
+    return NULL;
+
+  return g_build_filename (path, app_dir, NULL);
+}
+
 /**
  * ephy_web_application_get_profile_directory:
  * @id: the application identifier
@@ -167,18 +181,19 @@ get_app_id_from_profile_directory (const char *profile_dir)
 char *
 ephy_web_application_get_profile_directory (const char *id)
 {
-  char *dot_dir, *app_dir, *profile_dir;
+  return ephy_web_application_get_directory_under (id, g_get_user_data_dir ());
+}
 
-  app_dir = get_app_profile_directory_name (id);
-  if (!app_dir)
-    return NULL;
+static char *
+ephy_web_application_get_cache_directory (const char *id)
+{
+  return ephy_web_application_get_directory_under (id, g_get_user_cache_dir ());
+}
 
-  dot_dir = !ephy_profile_dir_is_default () ? ephy_default_profile_dir () : NULL;
-  profile_dir = g_build_filename (dot_dir ? dot_dir : g_get_user_data_dir (), app_dir, NULL);
-  g_free (app_dir);
-  g_free (dot_dir);
-
-  return profile_dir;
+static char *
+ephy_web_application_get_config_directory (const char *id)
+{
+  return ephy_web_application_get_directory_under (id, g_get_user_config_dir ());
 }
 
 /**
@@ -193,51 +208,68 @@ ephy_web_application_get_profile_directory (const char *id)
 gboolean
 ephy_web_application_delete (const char *id)
 {
-  char *profile_dir;
-  char *desktop_file = NULL, *desktop_path = NULL;
-  GFile *launcher = NULL;
-  gboolean return_value = FALSE;
+  g_autofree char *profile_dir = NULL;
+  g_autofree char *cache_dir = NULL;
+  g_autofree char *config_dir = NULL;
+  g_autofree char *desktop_file = NULL;
+  g_autofree char *desktop_path = NULL;
+  g_autoptr(GFile) launcher = NULL;
+  g_autoptr(GError) error = NULL;
 
   g_assert (id);
 
   profile_dir = ephy_web_application_get_profile_directory (id);
   if (!profile_dir)
-    goto out;
+    return FALSE;
 
   /* If there's no profile dir for this app, it means it does not
    * exist. */
   if (!g_file_test (profile_dir, G_FILE_TEST_IS_DIR)) {
     g_warning ("No application with id '%s' is installed.\n", id);
-    goto out;
+    return FALSE;
   }
 
-  if (!ephy_file_delete_dir_recursively (profile_dir, NULL))
-    goto out;
+  if (!ephy_file_delete_dir_recursively (profile_dir, &error)) {
+    g_warning ("Failed to recursively delete %s: %s", profile_dir, error->message);
+    return FALSE;
+  }
   LOG ("Deleted application profile.\n");
 
+  cache_dir = ephy_web_application_get_cache_directory (id);
+  if (g_file_test (cache_dir, G_FILE_TEST_IS_DIR)) {
+    if (!ephy_file_delete_dir_recursively (cache_dir, &error)) {
+      g_warning ("Failed to recursively delete %s: %s", cache_dir, error->message);
+      return FALSE;
+    }
+    LOG ("Deleted application cache directory.\n");
+  }
+
+  config_dir = ephy_web_application_get_config_directory (id);
+  if (g_file_test (config_dir, G_FILE_TEST_IS_DIR)) {
+    if (!ephy_file_delete_dir_recursively (config_dir, &error)) {
+      g_warning ("Failed to recursively delete %s: %s", config_dir, error->message);
+      return FALSE;
+    }
+    LOG ("Deleted application config directory.\n");
+  }
+
   desktop_file = get_app_desktop_filename (id);
-  if (!desktop_file)
-    goto out;
+  if (!desktop_file) {
+    g_warning ("Failed to compute desktop filename for app %s", id);
+    return FALSE;
+  }
 
   desktop_path = g_build_filename (g_get_user_data_dir (), "applications", desktop_file, NULL);
   if (g_file_test (desktop_path, G_FILE_TEST_IS_SYMLINK)) {
     launcher = g_file_new_for_path (desktop_path);
-    if (!g_file_delete (launcher, NULL, NULL))
-      goto out;
+    if (!g_file_delete (launcher, NULL, &error)) {
+      g_warning ("Failed to delete %s: %s", desktop_path, error->message);
+      return FALSE;
+    }
     LOG ("Deleted application launcher.\n");
   }
-  return_value = TRUE;
 
- out:
-
-  g_free (profile_dir);
-
-  if (launcher)
-    g_object_unref (launcher);
-  g_free (desktop_file);
-  g_free (desktop_path);
-
-  return return_value;
+  return TRUE;
 }
 
 static char *
@@ -359,6 +391,9 @@ ephy_web_application_create (const char *id,
     goto out;
   }
 
+  /* Skip migration for new web apps. */
+  ephy_profile_utils_set_migration_version_for_profile_dir (EPHY_PROFILE_MIGRATION_VERSION, profile_dir);
+
   /* Create an .app file. */
   g_autofree char *app_file = g_build_filename (profile_dir, ".app", NULL);
   int fd = g_open (app_file, O_WRONLY|O_CREAT|O_TRUNC, 0644);
@@ -563,22 +598,20 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
   GFileEnumerator *children = NULL;
   GFileInfo *info;
   GList *applications = NULL;
-  GFile *dot_dir;
-  char *default_dot_dir;
-  g_autofree char *profile_base = NULL;
+  g_autofree char *parent_directory_path = NULL;
+  g_autoptr(GFile) parent_directory = NULL;
 
-  default_dot_dir = !ephy_profile_dir_is_default () ? ephy_default_profile_dir () : NULL;
-  if (only_legacy) {
-    profile_base = g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
-    dot_dir = g_file_new_for_path (profile_base);
-  } else {
-    profile_base = g_strdup (g_get_user_data_dir ());
-    dot_dir = g_file_new_for_path (default_dot_dir ? default_dot_dir : g_get_user_data_dir ());
-  }
-  children = g_file_enumerate_children (dot_dir,
+  if (only_legacy)
+    parent_directory_path = g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
+  else
+    parent_directory_path = g_strdup (g_get_user_data_dir ());
+
+  parent_directory = g_file_new_for_path (parent_directory_path);
+  children = g_file_enumerate_children (parent_directory,
                                         "standard::name",
                                         0, NULL, NULL);
-  g_object_unref (dot_dir);
+  if (!children)
+    return NULL;
 
   info = g_file_enumerator_next_file (children, NULL, NULL);
   while (info) {
@@ -590,7 +623,7 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
       EphyWebApplication *app;
       char *profile_dir;
 
-      profile_dir = g_build_filename (profile_base, name, NULL);
+      profile_dir = g_build_filename (parent_directory_path, name, NULL);
       app = ephy_web_application_for_profile_directory (profile_dir);
       if (app) {
         if (!only_legacy) {
@@ -612,7 +645,6 @@ ephy_web_application_get_application_list_internal (gboolean only_legacy)
   }
 
   g_object_unref (children);
-  g_free (default_dot_dir);
 
   return g_list_reverse (applications);
 }
