@@ -91,7 +91,7 @@ query_async_data_new (EphyPasswordManagerQueryCallback callback,
 {
   QueryAsyncData *data;
 
-  data = g_slice_new (QueryAsyncData);
+  data = g_new (QueryAsyncData, 1);
   data->callback = callback;
   data->user_data = user_data;
 
@@ -103,7 +103,7 @@ query_async_data_free (QueryAsyncData *data)
 {
   g_assert (data);
 
-  g_slice_free (QueryAsyncData, data);
+  g_free (data);
 }
 
 static UpdatePasswordAsyncData *
@@ -112,7 +112,7 @@ update_password_async_data_new (EphyPasswordManager *manager,
 {
   UpdatePasswordAsyncData *data;
 
-  data = g_slice_new (UpdatePasswordAsyncData);
+  data = g_new (UpdatePasswordAsyncData, 1);
   data->manager = g_object_ref (manager);
   data->password = g_strdup (password);
 
@@ -126,7 +126,7 @@ update_password_async_data_free (UpdatePasswordAsyncData *data)
 
   g_object_unref (data->manager);
   g_free (data->password);
-  g_slice_free (UpdatePasswordAsyncData, data);
+  g_free (data);
 }
 
 static MergePasswordsAsyncData *
@@ -139,7 +139,7 @@ merge_passwords_async_data_new (EphyPasswordManager                    *manager,
 {
   MergePasswordsAsyncData *data;
 
-  data = g_slice_new (MergePasswordsAsyncData);
+  data = g_new (MergePasswordsAsyncData, 1);
   data->manager = g_object_ref (manager);
   data->is_initial = is_initial;
   data->remotes_deleted = remotes_deleted;
@@ -156,7 +156,7 @@ merge_passwords_async_data_free (MergePasswordsAsyncData *data)
   g_assert (data);
 
   g_object_unref (data->manager);
-  g_slice_free (MergePasswordsAsyncData, data);
+  g_free (data);
 }
 
 static ManageRecordAsyncData *
@@ -165,7 +165,7 @@ manage_record_async_data_new (EphyPasswordManager *manager,
 {
   ManageRecordAsyncData *data;
 
-  data = g_slice_new (ManageRecordAsyncData);
+  data = g_new (ManageRecordAsyncData, 1);
   data->manager = g_object_ref (manager);
   data->record = g_object_ref (record);
 
@@ -179,7 +179,7 @@ manage_record_async_data_free (ManageRecordAsyncData *data)
 
   g_object_unref (data->manager);
   g_object_unref (data->record);
-  g_slice_free (ManageRecordAsyncData, data);
+  g_free (data);
 }
 
 static GHashTable *
@@ -443,12 +443,14 @@ update_password_cb (GList    *records,
   EphyPasswordRecord *record;
 
   /* We expect only one matching record here. */
-  g_assert (g_list_length (records) == 1);
-
-  record = EPHY_PASSWORD_RECORD (records->data);
-  ephy_password_record_set_password (record, data->password);
-  ephy_password_manager_store_record (data->manager, record);
-  g_signal_emit_by_name (data->manager, "synchronizable-modified", record, FALSE);
+  if (g_list_length (records) == 1) {
+    record = EPHY_PASSWORD_RECORD (records->data);
+    ephy_password_record_set_password (record, data->password);
+    ephy_password_manager_store_record (data->manager, record);
+    g_signal_emit_by_name (data->manager, "synchronizable-modified", record, FALSE);
+  } else {
+    g_warn_if_reached ();
+  }
 
   g_list_free_full (records, g_object_unref);
   update_password_async_data_free (data);
@@ -472,9 +474,15 @@ ephy_password_manager_save (EphyPasswordManager *self,
   g_assert (EPHY_IS_PASSWORD_MANAGER (self));
   g_assert (origin);
   g_assert (target_origin);
+
+  /* Although we don't enforce these requirements for Firefox passwords,
+   * any password saved by Epiphany must have both password and
+   * password_field, so that it can be filled in. username and
+   * username_field are optional, but if one is provided, both must be.
+   */
   g_assert (password);
-  g_assert (!username_field || username);
-  g_assert (!password_field || password);
+  g_assert (password_field);
+  g_assert ((username && username_field) || (!username && !username_field));
 
   if (!is_new) {
     LOG ("Updating password for (%s, %s, %s, %s, %s)",
@@ -661,11 +669,13 @@ forget_cb (GList    *records,
   EphyPasswordRecord *record;
 
   /* We expect only one matching record here. */
-  g_assert (g_list_length (records) == 1);
-
-  record = EPHY_PASSWORD_RECORD (records->data);
-  g_signal_emit_by_name (self, "synchronizable-deleted", record);
-  ephy_password_manager_forget_record (self, record, NULL);
+  if (g_list_length (records) == 1) {
+    record = EPHY_PASSWORD_RECORD (records->data);
+    g_signal_emit_by_name (self, "synchronizable-deleted", record);
+    ephy_password_manager_forget_record (self, record, NULL);
+  } else {
+    g_warn_if_reached ();
+  }
 
   g_list_free_full (records, g_object_unref);
 }
@@ -782,9 +792,10 @@ replace_existing_cb (GList    *records,
   ManageRecordAsyncData *data = (ManageRecordAsyncData *)user_data;
 
   /* We expect only one matching record here. */
-  g_assert (g_list_length (records) == 1);
-
-  ephy_password_manager_forget_record (data->manager, records->data, data->record);
+  if (g_list_length (records) == 1)
+    ephy_password_manager_forget_record (data->manager, records->data, data->record);
+  else
+    g_warn_if_reached ();
 
   g_list_free_full (records, g_object_unref);
   manage_record_async_data_free (data);
