@@ -30,9 +30,7 @@
 #include <libsoup/soup.h>
 #include <nettle/aes.h>
 #include <nettle/cbc.h>
-#if (NETTLE_VERSION_MAJOR >= 3 && NETTLE_VERSION_MINOR >= 4)
 #include <nettle/hkdf.h>
-#endif
 #include <nettle/hmac.h>
 #include <string.h>
 
@@ -53,7 +51,7 @@ ephy_sync_crypto_hawk_options_new (const char *app,
 {
   SyncCryptoHawkOptions *options;
 
-  options = g_slice_new (SyncCryptoHawkOptions);
+  options = g_new (SyncCryptoHawkOptions, 1);
   options->app = g_strdup (app);
   options->dlg = g_strdup (dlg);
   options->ext = g_strdup (ext);
@@ -82,7 +80,7 @@ ephy_sync_crypto_hawk_options_free (SyncCryptoHawkOptions *options)
   g_free (options->payload);
   g_free (options->timestamp);
 
-  g_slice_free (SyncCryptoHawkOptions, options);
+  g_free (options);
 }
 
 static SyncCryptoHawkArtifacts *
@@ -99,7 +97,7 @@ ephy_sync_crypto_hawk_artifacts_new (const char *app,
 {
   SyncCryptoHawkArtifacts *artifacts;
 
-  artifacts = g_slice_new (SyncCryptoHawkArtifacts);
+  artifacts = g_new (SyncCryptoHawkArtifacts, 1);
   artifacts->app = g_strdup (app);
   artifacts->dlg = g_strdup (dlg);
   artifacts->ext = g_strdup (ext);
@@ -130,7 +128,7 @@ ephy_sync_crypto_hawk_artifacts_free (SyncCryptoHawkArtifacts *artifacts)
   g_free (artifacts->resource);
   g_free (artifacts->ts);
 
-  g_slice_free (SyncCryptoHawkArtifacts, artifacts);
+  g_free (artifacts);
 }
 
 static char *
@@ -383,7 +381,7 @@ ephy_sync_crypto_hawk_header_new (const char            *url,
       header = hawk_append_to_header (header, "dlg", artifacts->dlg);
   }
 
-  hawk_header = g_slice_new (SyncCryptoHawkHeader);
+  hawk_header = g_new (SyncCryptoHawkHeader, 1);
   hawk_header->header = g_strdup (header);
   hawk_header->artifacts = artifacts;
 
@@ -405,7 +403,7 @@ ephy_sync_crypto_hawk_header_free (SyncCryptoHawkHeader *header)
   g_free (header->header);
   ephy_sync_crypto_hawk_artifacts_free (header->artifacts);
 
-  g_slice_free (SyncCryptoHawkHeader, header);
+  g_free (header);
 }
 
 SyncCryptoRSAKeyPair *
@@ -429,7 +427,7 @@ ephy_sync_crypto_rsa_key_pair_new (void)
   /* Given correct parameters, this never fails. */
   g_assert (success);
 
-  key_pair = g_slice_new (SyncCryptoRSAKeyPair);
+  key_pair = g_new (SyncCryptoRSAKeyPair, 1);
   key_pair->public = public;
   key_pair->private = private;
 
@@ -444,7 +442,7 @@ ephy_sync_crypto_rsa_key_pair_free (SyncCryptoRSAKeyPair *key_pair)
   rsa_public_key_clear (&key_pair->public);
   rsa_private_key_clear (&key_pair->private);
 
-  g_slice_free (SyncCryptoRSAKeyPair, key_pair);
+  g_free (key_pair);
 }
 
 SyncCryptoKeyBundle *
@@ -465,7 +463,7 @@ ephy_sync_crypto_key_bundle_new (const char *aes_key_b64,
   hmac_key = g_base64_decode (hmac_key_b64, &hmac_key_len);
   g_assert (hmac_key_len == 32);
 
-  bundle = g_slice_new (SyncCryptoKeyBundle);
+  bundle = g_new (SyncCryptoKeyBundle, 1);
   bundle->aes_key_hex = ephy_sync_utils_encode_hex (aes_key, aes_key_len);
   bundle->hmac_key_hex = ephy_sync_utils_encode_hex (hmac_key, hmac_key_len);
 
@@ -483,7 +481,7 @@ ephy_sync_crypto_key_bundle_free (SyncCryptoKeyBundle *bundle)
   g_free (bundle->aes_key_hex);
   g_free (bundle->hmac_key_hex);
 
-  g_slice_free (SyncCryptoKeyBundle, bundle);
+  g_free (bundle);
 }
 
 static char *
@@ -532,7 +530,6 @@ ephy_sync_crypto_hkdf (const guint8 *in,
                        gsize         info_len,
                        gsize         out_len)
 {
-#if (NETTLE_VERSION_MAJOR >= 3 && NETTLE_VERSION_MINOR >= 4)
   struct hmac_sha256_ctx ctx;
   guint8 *salt;
   guint8 *prk;
@@ -563,68 +560,6 @@ ephy_sync_crypto_hkdf (const guint8 *in,
   g_free (prk);
 
   return out;
-#else
-  char *prk_hex;
-  char *tmp_hex;
-  guint8 *tmp;
-  guint8 *out_full;
-  guint8 *data;
-  guint8 *salt;
-  guint8 *prk;
-  guint8 *out;
-  guint8 counter;
-  gsize data_len;
-  gsize n;
-
-  g_assert (in);
-  g_assert (info);
-
-  /* Salt is an array of hash length zeros. */
-  salt = g_malloc0 (SHA256_DIGEST_SIZE);
-  out = g_malloc (out_len);
-
-  /* Step 1: Extract (https://tools.ietf.org/html/rfc5869) */
-  prk_hex = g_compute_hmac_for_data (G_CHECKSUM_SHA256,
-                                     salt, SHA256_DIGEST_SIZE,
-                                     in, in_len);
-  prk = ephy_sync_utils_decode_hex (prk_hex);
-
-  /* Step 2: Expand (https://tools.ietf.org/html/rfc5869) */
-  counter = 1;
-  n = (out_len + SHA256_DIGEST_SIZE - 1) / SHA256_DIGEST_SIZE;
-  out_full = g_malloc (n * SHA256_DIGEST_SIZE);
-
-  for (gsize i = 0; i < n; i++, counter++) {
-    if (i == 0) {
-      data = ephy_sync_crypto_concat_bytes (info, info_len, &counter, 1, NULL);
-      data_len = info_len + 1;
-    } else {
-      data = ephy_sync_crypto_concat_bytes (out_full + (i - 1) * SHA256_DIGEST_SIZE,
-                                            SHA256_DIGEST_SIZE, info, info_len,
-                                            &counter, 1, NULL);
-      data_len = SHA256_DIGEST_SIZE + info_len + 1;
-    }
-
-    tmp_hex = g_compute_hmac_for_data (G_CHECKSUM_SHA256,
-                                       prk, SHA256_DIGEST_SIZE,
-                                       data, data_len);
-    tmp = ephy_sync_utils_decode_hex (tmp_hex);
-    memcpy (out_full + i * SHA256_DIGEST_SIZE, tmp, SHA256_DIGEST_SIZE);
-
-    g_free (data);
-    g_free (tmp);
-    g_free (tmp_hex);
-  }
-
-  memcpy (out, out_full, out_len);
-
-  g_free (prk_hex);
-  g_free (salt);
-  g_free (prk);
-  g_free (out_full);
-
-  return out;
-#endif
 }
 
 void
@@ -855,7 +790,7 @@ ephy_sync_crypto_derive_master_bundle (const guint8 *key)
                                           prk, len,
                                           tmp, len + strlen (info) + 1);
 
-  bundle = g_slice_new (SyncCryptoKeyBundle);
+  bundle = g_new (SyncCryptoKeyBundle, 1);
   bundle->aes_key_hex = g_strdup (aes_key_hex);
   bundle->hmac_key_hex = g_strdup (hmac_key_hex);
 
