@@ -31,7 +31,7 @@ G_BEGIN_DECLS
 
 #define EPHY_TYPE_LOCATION_ENTRY (ephy_location_entry_get_type())
 
-G_DECLARE_FINAL_TYPE (EphyLocationEntry, ephy_location_entry, EPHY, LOCATION_ENTRY, GtkEntry)
+G_DECLARE_FINAL_TYPE (EphyLocationEntry, ephy_location_entry, EPHY, LOCATION_ENTRY, GtkOverlay)
 
 typedef enum {
   EPHY_LOCATION_ENTRY_BOOKMARK_ICON_HIDDEN,
@@ -41,21 +41,6 @@ typedef enum {
 
 GtkWidget      *ephy_location_entry_new                        (void);
 
-void            ephy_location_entry_set_completion             (EphyLocationEntry *entry,
-                                                                GtkTreeModel      *model,
-                                                                guint              text_col,
-                                                                guint              action_col,
-                                                                guint              keywords_col,
-                                                                guint              relevance_col,
-                                                                guint              url_col,
-                                                                guint              extra_col,
-                                                                guint              favicon_col);
-
-void            ephy_location_entry_set_match_func             (EphyLocationEntry           *entry,
-                                                                GtkEntryCompletionMatchFunc  match_func,
-                                                                gpointer                     user_data,
-                                                                GDestroyNotify               notify);
-
 gboolean        ephy_location_entry_get_can_undo               (EphyLocationEntry *entry);
 
 gboolean        ephy_location_entry_get_can_redo               (EphyLocationEntry *entry);
@@ -66,7 +51,7 @@ gboolean        ephy_location_entry_reset                      (EphyLocationEntr
 
 void            ephy_location_entry_undo_reset                 (EphyLocationEntry *entry);
 
-void            ephy_location_entry_activate                   (EphyLocationEntry *entry);
+void            ephy_location_entry_focus                      (EphyLocationEntry *entry);
 
 void            ephy_location_entry_set_bookmark_icon_state    (EphyLocationEntry                  *entry,
                                                                 EphyLocationEntryBookmarkIconState  state);
@@ -79,4 +64,22 @@ void            ephy_location_entry_set_add_bookmark_popover   (EphyLocationEntr
 
 GtkPopover     *ephy_location_entry_get_add_bookmark_popover   (EphyLocationEntry *entry);
 
+GtkWidget      *ephy_location_entry_get_entry                  (EphyLocationEntry *entry);
+
+GtkWidget      *ephy_location_entry_get_bookmark_widget        (EphyLocationEntry *entry);
+
+GtkWidget      *ephy_location_entry_get_reader_mode_widget     (EphyLocationEntry *entry);
+
+void            ephy_location_entry_set_reader_mode_visible    (EphyLocationEntry *entry,
+                                                                gboolean           visible);
+
+void            ephy_location_entry_set_reader_mode_state      (EphyLocationEntry *entry,
+                                                                gboolean           active);
+
+gboolean        ephy_location_entry_get_reader_mode_state      (EphyLocationEntry *entry);
+
+void            ephy_location_entry_set_progress               (EphyLocationEntry *entry,
+                                                                gdouble            progress,
+                                                                gboolean           loading);
+
 G_END_DECLS
