@@ -23,29 +23,18 @@
 #include "config.h"
 #include "ephy-header-bar.h"
 
-#include "ephy-action-helper.h"
 #include "ephy-add-bookmark-popover.h"
-#include "ephy-bookmarks-popover.h"
-#include "ephy-bookmark-properties-grid.h"
-#include "ephy-embed.h"
-#include "ephy-embed-container.h"
-#include "ephy-embed-prefs.h"
 #include "ephy-embed-utils.h"
-#include "ephy-favicon-helpers.h"
 #include "ephy-flatpak-utils.h"
-#include "ephy-gui.h"
-#include "ephy-history-service.h"
 #include "ephy-location-entry.h"
-#include "ephy-middle-clickable-button.h"
-#include "ephy-prefs.h"
-#include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-title-box.h"
 #include "ephy-title-widget.h"
 #include "ephy-type-builtins.h"
 
 #include <glib/gi18n.h>
-#include <webkit2/webkit2.h>
+#define HANDY_USE_UNSTABLE_API
+#include <handy.h>
 
 enum {
   PROP_0,
@@ -67,7 +56,6 @@ struct _EphyHeaderBar {
   GtkWidget *navigation_box;
   GtkWidget *reader_mode_revealer;
   GtkWidget *reader_mode_button;
-  GtkWidget *new_tab_revealer;
   GtkWidget *new_tab_button;
   GtkWidget *bookmarks_button;
   GtkWidget *page_menu_button;
@@ -123,8 +111,6 @@ sync_chromes_visibility (EphyHeaderBar *header_bar)
   ephy_action_bar_end_set_show_bookmarks_button (header_bar->action_bar_end,
                                                  chrome & EPHY_WINDOW_CHROME_BOOKMARKS);
   gtk_widget_set_visible (header_bar->page_menu_button, chrome & EPHY_WINDOW_CHROME_MENU);
-  ephy_action_bar_end_set_show_new_tab_button (header_bar->action_bar_end,
-                                               chrome & EPHY_WINDOW_CHROME_TABSBAR);
 }
 
 static void
@@ -149,6 +135,17 @@ update_revealer_visibility (GtkRevealer *revealer)
                           gtk_revealer_get_child_revealed (revealer));
 }
 
+static gboolean
+is_desktop_pantheon (void)
+{
+  const gchar *xdg_current_desktop = g_environ_getenv (g_get_environ (), "XDG_CURRENT_DESKTOP");
+
+  if (!xdg_current_desktop)
+      return FALSE;
+
+  return strstr (xdg_current_desktop, "Pantheon") != NULL;
+}
+
 static void
 ephy_header_bar_constructed (GObject *object)
 {
@@ -157,11 +154,13 @@ ephy_header_bar_constructed (GObject *object)
   GtkWidget *page_menu_popover;
   GtkBuilder *builder;
   EphyEmbedShell *embed_shell;
+  HdyColumn *column;
 
   G_OBJECT_CLASS (ephy_header_bar_parent_class)->constructed (object);
 
-  g_signal_connect_swapped (header_bar->window, "notify::chrome",
-                            G_CALLBACK (sync_chromes_visibility), header_bar);
+  g_signal_connect_object (header_bar->window, "notify::chrome",
+                           G_CALLBACK (sync_chromes_visibility), header_bar,
+                           G_CONNECT_SWAPPED);
 
   /* Start action elements */
   header_bar->action_bar_start = ephy_action_bar_start_new ();
@@ -173,7 +172,6 @@ ephy_header_bar_constructed (GObject *object)
                     G_CALLBACK (update_revealer_visibility), NULL);
   gtk_revealer_set_transition_type (GTK_REVEALER (header_bar->start_revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT);
   gtk_container_add (GTK_CONTAINER (header_bar->start_revealer), GTK_WIDGET (header_bar->action_bar_start));
-  gtk_widget_set_margin_end (GTK_WIDGET (header_bar->action_bar_start), 54);
 
   gtk_header_bar_pack_start (GTK_HEADER_BAR (header_bar),
                              GTK_WIDGET (header_bar->start_revealer));
@@ -185,7 +183,13 @@ ephy_header_bar_constructed (GObject *object)
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_title_box_new ());
   else
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_location_entry_new ());
-  gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), GTK_WIDGET (header_bar->title_widget));
+  column = hdy_column_new ();
+  gtk_widget_set_hexpand (GTK_WIDGET (column), TRUE);
+  gtk_widget_show (GTK_WIDGET (column));
+  hdy_column_set_maximum_width (column, 860);
+  hdy_column_set_linear_growth_width (column, 560);
+  gtk_container_add (GTK_CONTAINER (column), GTK_WIDGET (header_bar->title_widget));
+  gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), GTK_WIDGET (column));
   gtk_widget_show (GTK_WIDGET (header_bar->title_widget));
 
   if (EPHY_IS_LOCATION_ENTRY (header_bar->title_widget)) {
@@ -209,10 +213,33 @@ ephy_header_bar_constructed (GObject *object)
   builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/gtk/page-menu-popover.ui");
   page_menu_popover = GTK_WIDGET (gtk_builder_get_object (builder, "page-menu-popover"));
   header_bar->zoom_level_button = GTK_WIDGET (gtk_builder_get_object (builder, "zoom-level"));
-  if (ephy_is_running_inside_flatpak ()) {
+  if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "new-window-separator")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "new-window-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "new-incognito-window-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "reopen-closed-tab-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "bookmarks-separator")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "import-bookmarks-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "export-bookmarks-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "save-as-application-separator")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "save-as-application-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "application-manager-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "override-text-encoding-separator")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "override-text-encoding-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "keyboard-shortcuts-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "help-button")));
+  } else if (ephy_is_running_inside_flatpak ()) {
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "save-as-application-separator")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "save-as-application-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "application-manager-button")));
+
+    if (is_desktop_pantheon ())
+      gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "help-button")));
   }
+
+  if (is_desktop_pantheon ())
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "about-button")));
+
   gtk_menu_button_set_popover (GTK_MENU_BUTTON (button), page_menu_popover);
   g_object_unref (builder);
 
@@ -228,7 +255,6 @@ ephy_header_bar_constructed (GObject *object)
                     G_CALLBACK (update_revealer_visibility), NULL);
   gtk_revealer_set_transition_type (GTK_REVEALER (header_bar->end_revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_LEFT);
   gtk_container_add (GTK_CONTAINER (header_bar->end_revealer), GTK_WIDGET (header_bar->action_bar_end));
-  gtk_widget_set_margin_start (GTK_WIDGET (header_bar->action_bar_end), 54);
 
   gtk_header_bar_pack_end (GTK_HEADER_BAR (header_bar),
                            GTK_WIDGET (header_bar->end_revealer));
