@@ -29,6 +29,7 @@
 #include "ephy-embed.h"
 #include "ephy-file-helpers.h"
 #include "ephy-link.h"
+#include "ephy-pages-popover.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
@@ -48,6 +49,8 @@
 struct _EphyNotebook {
   GtkNotebook parent_instance;
 
+  EphyAdaptiveMode adaptive_mode;
+
   GList *focused_pages;
   guint tabs_vis_notifier_id;
 
@@ -253,6 +256,7 @@ ephy_notebook_switch_page_cb (GtkNotebook *notebook,
   GtkWidget *child;
 
   child = gtk_notebook_get_nth_page (notebook, page_num);
+  gtk_widget_grab_focus (child);
 
   /* Remove the old page, we dont want to grow unnecessarily
    * the list */
@@ -344,10 +348,12 @@ static void
 update_tabs_visibility (EphyNotebook *nb,
                         gboolean      before_inserting)
 {
+  EphyEmbedShellMode mode;
   gboolean show_tabs = FALSE;
   guint num;
   EphyPrefsUITabsBarVisibilityPolicy policy;
 
+  mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (ephy_shell_get_default ()));
   num = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
 
   if (before_inserting)
@@ -356,8 +362,10 @@ update_tabs_visibility (EphyNotebook *nb,
   policy = g_settings_get_enum (EPHY_SETTINGS_UI,
                                 EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY);
 
-  if (((policy == EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_MORE_THAN_ONE && num > 1) ||
-       policy == EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_ALWAYS))
+  if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION &&
+      nb->adaptive_mode != EPHY_ADAPTIVE_MODE_NARROW &&
+      ((policy == EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_MORE_THAN_ONE && num > 1) ||
+        policy == EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_ALWAYS))
     show_tabs = TRUE;
 
   /* Only show the tabs when the "tabs-allowed" property is TRUE. */
@@ -471,6 +479,7 @@ ephy_notebook_constructed (GObject *object)
   EphyNotebook *notebook = EPHY_NOTEBOOK (object);
   GtkWidget *hbox;
   GtkWidget *button;
+  EphyPagesPopover *popover;
 
   G_OBJECT_CLASS (ephy_notebook_parent_class)->constructed (object);
 
@@ -478,14 +487,6 @@ ephy_notebook_constructed (GObject *object)
   gtk_notebook_set_action_widget (GTK_NOTEBOOK (notebook), hbox, GTK_PACK_END);
   gtk_widget_show (hbox);
 
-  button = gtk_button_new_from_icon_name ("tab-new-symbolic", GTK_ICON_SIZE_BUTTON);
-  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-  gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "win.new-tab");
-  /* Translators: tooltip for the new tab button */
-  gtk_widget_set_tooltip_text (button, _("Open a new tab"));
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
-
   button = gtk_menu_button_new ();
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   /* Translators: tooltip for the tab switcher menu button */
@@ -495,8 +496,11 @@ ephy_notebook_constructed (GObject *object)
 
   notebook->tab_menu = g_menu_new ();
   /* Remove this when popover menus become scrollable. */
-  gtk_menu_button_set_use_popover (GTK_MENU_BUTTON (button), FALSE);
-  gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (button), G_MENU_MODEL (notebook->tab_menu));
+  gtk_menu_button_set_use_popover (GTK_MENU_BUTTON (button), TRUE);
+
+  popover = ephy_pages_popover_new (GTK_WIDGET (button));
+  ephy_pages_popover_set_notebook (popover, notebook);
+  gtk_menu_button_set_popover (GTK_MENU_BUTTON (button), GTK_WIDGET (popover));
 }
 
 static void
@@ -1044,3 +1048,21 @@ ephy_notebook_prev_page (EphyNotebook *notebook)
       gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), -1);
   }
 }
+
+GMenu *
+ephy_notebook_get_pages_menu (EphyNotebook *notebook)
+{
+  g_assert (EPHY_IS_NOTEBOOK (notebook));
+
+  return notebook->tab_menu;
+}
+
+void
+ephy_notebook_set_adaptive_mode (EphyNotebook     *notebook,
+                                 EphyAdaptiveMode  adaptive_mode)
+{
+  g_assert (EPHY_IS_NOTEBOOK (notebook));
+
+  notebook->adaptive_mode = adaptive_mode;
+  update_tabs_visibility (notebook, FALSE);
+}
