@@ -24,7 +24,6 @@
 #include "ephy-shell.h"
 
 #include "ephy-debug.h"
-#include "ephy-downloads-manager.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
@@ -46,7 +45,6 @@
 #include "window-commands.h"
 
 #include <glib/gi18n.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 struct _EphyShell {
@@ -57,7 +55,6 @@ struct _EphyShell {
   GList *windows;
   GObject *lockdown;
   EphyBookmarksManager *bookmarks_manager;
-  EphyPasswordManager *password_manager;
   EphyHistoryManager *history_manager;
   EphyOpenTabsManager *open_tabs_manager;
   GNetworkMonitor *network_monitor;
@@ -77,9 +74,8 @@ G_DEFINE_TYPE (EphyShell, ephy_shell, EPHY_TYPE_EMBED_SHELL)
 
 /**
  * ephy_shell_startup_context_new:
- * @bookmarks_filename: A bookmarks file to import.
+ * @startup_mode: An #EphyStartupMode (new tab or new window)
  * @session_filename: A session to restore.
- * @bookmark_url: A URL to be added to the bookmarks.
  * @arguments: A %NULL-terminated array of URLs and file URIs to be opened.
  * @user_time: The user time when the EphyShell startup was invoked.
  *
@@ -89,23 +85,16 @@ G_DEFINE_TYPE (EphyShell, ephy_shell, EPHY_TYPE_EMBED_SHELL)
  * Returns: a newly allocated #EphyShellStartupContext
  **/
 EphyShellStartupContext *
-ephy_shell_startup_context_new (EphyStartupFlags startup_flags,
-                                char *bookmarks_filename,
-                                char *session_filename,
-                                char *bookmark_url,
-                                char **arguments,
-                                guint32 user_time)
+ephy_shell_startup_context_new (EphyStartupMode   startup_mode,
+                                char             *session_filename,
+                                char            **arguments,
+                                guint32           user_time)
 {
-  EphyShellStartupContext *ctx = g_slice_new0 (EphyShellStartupContext);
+  EphyShellStartupContext *ctx = g_new0 (EphyShellStartupContext, 1);
 
-  ctx->startup_flags = startup_flags;
-
-  ctx->bookmarks_filename = g_strdup (bookmarks_filename);
+  ctx->startup_mode = startup_mode;
   ctx->session_filename = g_strdup (session_filename);
-  ctx->bookmark_url = g_strdup (bookmark_url);
-
   ctx->arguments = g_strdupv (arguments);
-
   ctx->user_time = user_time;
 
   return ctx;
@@ -116,17 +105,14 @@ ephy_shell_startup_context_free (EphyShellStartupContext *ctx)
 {
   g_assert (ctx != NULL);
 
-  g_free (ctx->bookmarks_filename);
   g_free (ctx->session_filename);
-  g_free (ctx->bookmark_url);
-
   g_strfreev (ctx->arguments);
-
-  g_slice_free (EphyShellStartupContext, ctx);
+  g_free (ctx);
 }
 
 static void
-ephy_shell_startup_continue (EphyShell *shell, EphyShellStartupContext *ctx)
+ephy_shell_startup_continue (EphyShell               *shell,
+                             EphyShellStartupContext *ctx)
 {
   EphySession *session = ephy_shell_get_session (shell);
 
@@ -138,7 +124,7 @@ ephy_shell_startup_continue (EphyShell *shell, EphyShellStartupContext *ctx)
     /* Don't queue any window openings if no extra arguments given, */
     /* since session autoresume will open one for us. */
     ephy_shell_open_uris (shell, (const char **)ctx->arguments,
-                          ctx->startup_flags, ctx->user_time);
+                          ctx->startup_mode, ctx->user_time);
   }
 }
 
@@ -271,46 +257,17 @@ static GActionEntry app_entries[] = {
   { "quit", quit_application, NULL, NULL, NULL },
 };
 
+static GActionEntry non_incognito_extra_app_entries[] = {
+  { "reopen-closed-tab", reopen_closed_tab, NULL, NULL, NULL },
+};
+
 static GActionEntry app_mode_app_entries[] = {
+  { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
 };
 
-static GActionEntry app_normal_mode_entries[] = {
-  { "reopen-closed-tab", reopen_closed_tab, NULL, NULL, NULL },
-};
-
-static void
-download_started_cb (WebKitWebContext *web_context,
-                     WebKitDownload   *download,
-                     EphyShell        *shell)
-{
-  EphyDownload *ephy_download;
-  gboolean ephy_download_set;
-
-  /* Is download locked down? */
-  if (g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-                              EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK)) {
-    webkit_download_cancel (download);
-    return;
-  }
-
-  /* Only create an EphyDownload for the WebKitDownload if it doesn't exist yet.
-   * This can happen when the download has been started automatically by WebKit,
-   * due to a context menu action or policy checker decision. Downloads started
-   * explicitly by Epiphany are marked with ephy-download-set GObject data.
-   */
-  ephy_download_set = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (download), "ephy-download-set"));
-  if (ephy_download_set)
-    return;
-
-  ephy_download = ephy_download_new (download);
-  ephy_downloads_manager_add_download (ephy_embed_shell_get_downloads_manager (EPHY_EMBED_SHELL (shell)),
-                                       ephy_download);
-  g_object_unref (ephy_download);
-}
-
 static void
 register_synchronizable_managers (EphyShell       *shell,
                                   EphySyncService *service)
@@ -331,7 +288,7 @@ register_synchronizable_managers (EphyShell       *shell,
   }
 
   if (ephy_sync_utils_passwords_sync_is_enabled ()) {
-    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_password_manager (shell));
+    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL (shell)));
     ephy_sync_service_register_manager (service, manager);
   }
 
@@ -377,26 +334,26 @@ sync_secrets_load_finished_cb (EphySyncService *service,
   ephy_sync_service_start_sync (service);
 }
 
+static void
+set_accel_for_action (EphyShell   *shell,
+                      const gchar *detailed_action_name,
+                      const gchar *accel)
+{
+  const char *accels[] = { accel, NULL };
+
+  gtk_application_set_accels_for_action (GTK_APPLICATION (shell), detailed_action_name, accels);
+}
+
 static void
 ephy_shell_startup (GApplication *application)
 {
   EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (application);
   EphyShell *shell = EPHY_SHELL (application);
   EphyEmbedShellMode mode;
-  GtkBuilder *builder;
 
   G_APPLICATION_CLASS (ephy_shell_parent_class)->startup (application);
 
   /* We're not remoting; start our services */
-  g_signal_connect (ephy_embed_shell_get_web_context (embed_shell),
-                    "download-started",
-                    G_CALLBACK (download_started_cb),
-                    application);
-
-  builder = gtk_builder_new ();
-  gtk_builder_add_from_resource (builder,
-                                 "/org/gnome/epiphany/gtk/application-menu.ui",
-                                 NULL);
 
   mode = ephy_embed_shell_get_mode (embed_shell);
   if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION) {
@@ -404,9 +361,10 @@ ephy_shell_startup (GApplication *application)
                                      app_entries, G_N_ELEMENTS (app_entries),
                                      application);
 
-    if (mode != EPHY_EMBED_SHELL_MODE_INCOGNITO) {
+    if (mode != EPHY_EMBED_SHELL_MODE_INCOGNITO &&
+        mode != EPHY_EMBED_SHELL_MODE_AUTOMATION) {
       g_action_map_add_action_entries (G_ACTION_MAP (application),
-                                       app_normal_mode_entries, G_N_ELEMENTS (app_normal_mode_entries),
+                                       non_incognito_extra_app_entries, G_N_ELEMENTS (non_incognito_extra_app_entries),
                                        application);
       g_object_bind_property (G_OBJECT (ephy_shell_get_session (shell)),
                               "can-undo-tab-closed",
@@ -419,21 +377,63 @@ ephy_shell_startup (GApplication *application)
         /* Create the sync service. */
         ephy_shell_get_sync_service (shell);
       }
-    }
 
-    gtk_application_set_app_menu (GTK_APPLICATION (application),
-                                  G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
+      /* Actions that are disabled in app mode */
+      set_accel_for_action (shell, "app.new-window", "<Primary>n");
+      set_accel_for_action (shell, "app.new-incognito", "<Primary><Shift>n");
+      set_accel_for_action (shell, "app.reopen-closed-tab", "<Primary><Shift>t");
+      set_accel_for_action (shell, "app.import-bookmarks", "<Primary><Shift>m");
+      set_accel_for_action (shell, "app.export-bookmarks", "<Primary><Shift>x");
+      set_accel_for_action (shell, "app.shortcuts", "<Primary>F1");
+      set_accel_for_action (shell, "app.help", "F1");
+    }
   } else {
     g_action_map_add_action_entries (G_ACTION_MAP (application),
                                      app_mode_app_entries, G_N_ELEMENTS (app_mode_app_entries),
                                      application);
-    gtk_application_set_app_menu (GTK_APPLICATION (application),
-                                  G_MENU_MODEL (gtk_builder_get_object (builder, "app-mode-app-menu")));
   }
 
-  g_object_unref (builder);
+  /* Actions that are available in both app mode and browser mode */
+  set_accel_for_action (shell, "app.history", "<Primary>h");
+  set_accel_for_action (shell, "app.preferences", "<Primary>e");
+  set_accel_for_action (shell, "app.quit", "<Primary>q");
 }
 
+static GtkWidget *
+create_web_view_for_automation_cb (WebKitAutomationSession *session,
+                                   EphyShell               *shell)
+{
+  EphyEmbed *embed;
+  EphyWindow *window;
+  EphyWebView *web_view;
+  guint n_embeds;
+
+  window = EPHY_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (shell)));
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  n_embeds = ephy_embed_container_get_n_children (EPHY_EMBED_CONTAINER (window));
+  web_view = ephy_embed_get_web_view (embed);
+  if (n_embeds == 1 && ephy_web_view_get_visit_type (web_view) == EPHY_PAGE_VISIT_HOMEPAGE)
+    return GTK_WIDGET (web_view);
+
+  embed = ephy_shell_new_tab (shell, window, NULL, EPHY_NEW_TAB_JUMP);
+  return GTK_WIDGET (ephy_embed_get_web_view (embed));
+}
+
+static void
+automation_started_cb (WebKitWebContext        *web_context,
+                       WebKitAutomationSession *session,
+                       EphyShell               *shell)
+{
+  WebKitApplicationInfo *info = webkit_application_info_new ();
+  webkit_application_info_set_name (info, "Epiphany");
+  webkit_application_info_set_version (info, EPHY_MAJOR_VERSION, EPHY_MINOR_VERSION, EPHY_MICRO_VERSION);
+  webkit_automation_session_set_application_info (session, info);
+  webkit_application_info_unref (info);
+
+  g_signal_connect (session, "create-web-view", G_CALLBACK (create_web_view_for_automation_cb), shell);
+}
+
+
 static void
 session_load_cb (GObject      *object,
                  GAsyncResult *result,
@@ -450,16 +450,20 @@ static void
 ephy_shell_activate (GApplication *application)
 {
   EphyShell *shell = EPHY_SHELL (application);
+  EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (shell);
+
+  if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_AUTOMATION) {
+    WebKitWebContext *web_context = ephy_embed_shell_get_web_context (embed_shell);
+    g_signal_connect (web_context, "automation-started", G_CALLBACK (automation_started_cb), shell);
+  }
 
-  /*
-   * We get here on each new instance (remote or not). Autoresume the
-   * session and queue the commands if we are a secondary instance. Otherwise,
-   * execute the commands immediately, before the remote startup context
-   * can be invalidated by another remote instance.
-   */
   if (shell->remote_startup_context == NULL) {
     EphySession *session = ephy_shell_get_session (shell);
 
+    /* We are activating the primary instance for the first time. If we
+     * have a saved session, resume it first, then run any startup
+     * commands in session_load_cb. Otherwise, run them now.
+     */
     if (session) {
       ephy_session_resume (session,
                            shell->local_startup_context->user_time,
@@ -467,6 +471,12 @@ ephy_shell_activate (GApplication *application)
     } else
       ephy_shell_startup_continue (shell, shell->local_startup_context);
   } else {
+    /* We are activating the primary instance in response to the launch
+     * of a secondary instance. Execute the commands immediately. We
+     * have to be careful because if we don't handle the commands
+     * immediately, the remote startup context could be invalidated by
+     * the launch of another remote instance.
+     */
     ephy_shell_startup_continue (shell, shell->remote_startup_context);
     g_clear_pointer (&shell->remote_startup_context, ephy_shell_startup_context_free);
   }
@@ -478,10 +488,8 @@ ephy_shell_activate (GApplication *application)
  * instance.
  */
 typedef enum {
-  CTX_STARTUP_FLAGS,
-  CTX_BOOKMARKS_FILENAME,
+  CTX_STARTUP_MODE,
   CTX_SESSION_FILENAME,
-  CTX_BOOKMARK_URL,
   CTX_ARGUMENTS,
   CTX_USER_TIME
 } CtxEnum;
@@ -509,26 +517,16 @@ ephy_shell_add_platform_data (GApplication    *application,
     ctx_builder = g_variant_builder_new (G_VARIANT_TYPE_ARRAY);
     ctx = app->local_startup_context;
 
-    if (ctx->startup_flags)
+    if (ctx->startup_mode)
       g_variant_builder_add (ctx_builder, "{iv}",
-                             CTX_STARTUP_FLAGS,
-                             g_variant_new_byte (ctx->startup_flags));
-
-    if (ctx->bookmarks_filename)
-      g_variant_builder_add (ctx_builder, "{iv}",
-                             CTX_BOOKMARKS_FILENAME,
-                             g_variant_new_string (ctx->bookmarks_filename));
+                             CTX_STARTUP_MODE,
+                             g_variant_new_byte (ctx->startup_mode));
 
     if (ctx->session_filename)
       g_variant_builder_add (ctx_builder, "{iv}",
                              CTX_SESSION_FILENAME,
                              g_variant_new_string (ctx->session_filename));
 
-    if (ctx->bookmark_url)
-      g_variant_builder_add (ctx_builder, "{iv}",
-                             CTX_BOOKMARK_URL,
-                             g_variant_new_string (ctx->bookmark_url));
-
     /*
      * If there are no URIs specified, pass an empty string, so that
      * the primary instance opens a new window.
@@ -569,7 +567,7 @@ ephy_shell_before_emit (GApplication *application,
   g_variant_iter_init (&iter, platform_data);
   while (g_variant_iter_loop (&iter, "{&sv}", &key, &value)) {
     if (strcmp (key, "ephy-shell-startup-context") == 0) {
-      ctx = g_slice_new0 (EphyShellStartupContext);
+      ctx = g_new0 (EphyShellStartupContext, 1);
 
       /*
        * Iterate over the startup context variant and fill the members
@@ -578,18 +576,12 @@ ephy_shell_before_emit (GApplication *application,
       g_variant_iter_init (&ctx_iter, value);
       while (g_variant_iter_loop (&ctx_iter, "{iv}", &ctx_key, &ctx_value)) {
         switch (ctx_key) {
-          case CTX_STARTUP_FLAGS:
-            ctx->startup_flags = g_variant_get_byte (ctx_value);
-            break;
-          case CTX_BOOKMARKS_FILENAME:
-            ctx->bookmarks_filename = g_variant_dup_string (ctx_value, NULL);
+          case CTX_STARTUP_MODE:
+            ctx->startup_mode = g_variant_get_byte (ctx_value);
             break;
           case CTX_SESSION_FILENAME:
             ctx->session_filename = g_variant_dup_string (ctx_value, NULL);
             break;
-          case CTX_BOOKMARK_URL:
-            ctx->bookmark_url = g_variant_dup_string (ctx_value, NULL);
-            break;
           case CTX_ARGUMENTS:
             ctx->arguments = g_variant_dup_strv (ctx_value, NULL);
             break;
@@ -634,6 +626,11 @@ ephy_shell_constructed (GObject *object)
     g_application_set_flags (G_APPLICATION (object), flags);
   }
 
+  if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (object)) == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    dzl_application_add_resources (DZL_APPLICATION (object),
+                                   "resource:///org/gnome/Epiphany");
+  }
+
   /* FIXME: not sure if this is the best place to put this stuff. */
   ephy_shell_get_lockdown (EPHY_SHELL (object));
 
@@ -689,7 +686,6 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&shell->network_monitor);
   g_clear_object (&shell->sync_service);
   g_clear_object (&shell->bookmarks_manager);
-  g_clear_object (&shell->password_manager);
   g_clear_object (&shell->history_manager);
   g_clear_object (&shell->open_tabs_manager);
 
@@ -738,13 +734,13 @@ ephy_shell_get_default (void)
  * Return value: (transfer none): the created #EphyEmbed
  **/
 EphyEmbed *
-ephy_shell_new_tab_full (EphyShell      *shell,
-                         const char     *title,
-                         WebKitWebView  *related_view,
-                         EphyWindow     *window,
-                         EphyEmbed      *previous_embed,
-                         EphyNewTabFlags flags,
-                         guint32         user_time)
+ephy_shell_new_tab_full (EphyShell       *shell,
+                         const char      *title,
+                         WebKitWebView   *related_view,
+                         EphyWindow      *window,
+                         EphyEmbed       *previous_embed,
+                         EphyNewTabFlags  flags,
+                         guint32          user_time)
 {
   EphyEmbedShell *embed_shell;
   GtkWidget *web_view;
@@ -758,7 +754,8 @@ ephy_shell_new_tab_full (EphyShell      *shell,
 
   embed_shell = EPHY_EMBED_SHELL (shell);
 
-  if (flags & EPHY_NEW_TAB_JUMP) jump_to = TRUE;
+  if (flags & EPHY_NEW_TAB_JUMP)
+    jump_to = TRUE;
 
   LOG ("Opening new tab window %p parent-embed %p jump-to:%s",
        window, previous_embed, jump_to ? "t" : "f");
@@ -784,6 +781,7 @@ ephy_shell_new_tab_full (EphyShell      *shell,
   embed = EPHY_EMBED (g_object_new (EPHY_TYPE_EMBED,
                                     "web-view", web_view,
                                     "title", title,
+                                    "progress-bar-enabled", ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION,
                                     NULL));
   gtk_widget_show (GTK_WIDGET (embed));
   ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, position, jump_to);
@@ -808,10 +806,10 @@ ephy_shell_new_tab_full (EphyShell      *shell,
  * Return value: (transfer none): the created #EphyEmbed
  **/
 EphyEmbed *
-ephy_shell_new_tab (EphyShell      *shell,
-                    EphyWindow     *parent_window,
-                    EphyEmbed      *previous_embed,
-                    EphyNewTabFlags flags)
+ephy_shell_new_tab (EphyShell       *shell,
+                    EphyWindow      *parent_window,
+                    EphyEmbed       *previous_embed,
+                    EphyNewTabFlags  flags)
 {
   return ephy_shell_new_tab_full (shell, NULL, NULL, parent_window,
                                   previous_embed, flags,
@@ -834,7 +832,7 @@ ephy_shell_get_session (EphyShell *shell)
   g_assert (EPHY_IS_SHELL (shell));
 
   mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell));
-  if (mode ==  EPHY_EMBED_SHELL_MODE_APPLICATION || mode == EPHY_EMBED_SHELL_MODE_INCOGNITO)
+  if (mode ==  EPHY_EMBED_SHELL_MODE_APPLICATION || mode == EPHY_EMBED_SHELL_MODE_INCOGNITO || mode == EPHY_EMBED_SHELL_MODE_AUTOMATION)
     return NULL;
 
   if (shell->session == NULL)
@@ -891,25 +889,6 @@ ephy_shell_get_bookmarks_manager (EphyShell *shell)
   return shell->bookmarks_manager;
 }
 
-/**
- * ephy_shell_get_password_manager:
- * @shell: the #EphyShell
- *
- * Returns the passwords manager.
- *
- * Return value: (transfer none): An #EphyPasswordManager.
- */
-EphyPasswordManager *
-ephy_shell_get_password_manager (EphyShell *shell)
-{
-  g_assert (EPHY_IS_SHELL (shell));
-
-  if (shell->password_manager == NULL)
-    shell->password_manager = ephy_password_manager_new ();
-
-  return shell->password_manager;
-}
-
 /**
  * ephy_shell_get_history_manager:
  * @shell: the #EphyShell
@@ -1102,38 +1081,30 @@ typedef struct {
 } OpenURIsData;
 
 static OpenURIsData *
-open_uris_data_new (EphyShell       *shell,
-                    const char     **uris,
-                    EphyStartupFlags startup_flags,
-                    guint32          user_time)
+open_uris_data_new (EphyShell        *shell,
+                    const char      **uris,
+                    EphyStartupMode   startup_mode,
+                    guint32           user_time)
 {
   OpenURIsData *data;
-  gboolean new_windows_in_tabs;
   gboolean fullscreen_lockdown;
-  gboolean have_uris;
   EphySession *session = ephy_shell_get_session (shell);
 
-  data = g_slice_new0 (OpenURIsData);
+  data = g_new0 (OpenURIsData, 1);
   data->shell = shell;
   data->session = session ? g_object_ref (session) : NULL;
   data->uris = g_strdupv ((char **)uris);
   data->user_time = user_time;
 
-  new_windows_in_tabs = g_settings_get_boolean (EPHY_SETTINGS_MAIN,
-                                                EPHY_PREFS_NEW_WINDOWS_IN_TABS);
   fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
                                                 EPHY_PREFS_LOCKDOWN_FULLSCREEN);
 
-  have_uris = uris && !(g_strv_length ((char **)uris) == 1 && !g_strcmp0 (uris[0], ""));
-
-  if (startup_flags & EPHY_STARTUP_NEW_WINDOW && !fullscreen_lockdown) {
+  if (startup_mode == EPHY_STARTUP_NEW_WINDOW && !fullscreen_lockdown) {
     data->window = ephy_window_new ();
-  } else if (startup_flags & EPHY_STARTUP_NEW_TAB || (new_windows_in_tabs && have_uris)) {
+  } else {
     data->flags |= EPHY_NEW_TAB_JUMP;
     data->window = EPHY_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (shell)));
     data->reuse_empty_tab = TRUE;
-  } else if (!have_uris) {
-    data->window = ephy_window_new ();
   }
 
   g_application_hold (G_APPLICATION (shell));
@@ -1147,8 +1118,7 @@ open_uris_data_free (OpenURIsData *data)
   g_application_release (G_APPLICATION (data->shell));
   g_clear_object (&data->session);
   g_strfreev (data->uris);
-
-  g_slice_free (OpenURIsData, data);
+  g_free (data);
 }
 
 static gboolean
@@ -1220,22 +1190,17 @@ ephy_shell_open_uris_idle_done (OpenURIsData *data)
 }
 
 void
-ephy_shell_open_uris (EphyShell       *shell,
-                      const char     **uris,
-                      EphyStartupFlags startup_flags,
-                      guint32          user_time)
+ephy_shell_open_uris (EphyShell        *shell,
+                      const char      **uris,
+                      EphyStartupMode   startup_mode,
+                      guint32           user_time)
 {
   OpenURIsData *data;
   guint id;
 
   g_assert (EPHY_IS_SHELL (shell));
 
-  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) == EPHY_EMBED_SHELL_MODE_APPLICATION) {
-    for (int i = 0; uris[i] != NULL; i++)
-      ephy_embed_shell_add_app_related_uri (ephy_embed_shell_get_default (), uris[i]);
-  }
-
-  data = open_uris_data_new (shell, uris, startup_flags, user_time);
+  data = open_uris_data_new (shell, uris, startup_mode, user_time);
   id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
                         (GSourceFunc)ephy_shell_open_uris_idle,
                         data,
