@@ -33,6 +33,7 @@
 #include "ephy-flatpak-utils.h"
 #include "ephy-gui.h"
 #include "ephy-langs.h"
+#include "ephy-lib-type-builtins.h"
 #include "ephy-prefs.h"
 #include "ephy-search-engine-dialog.h"
 #include "ephy-session.h"
@@ -43,16 +44,19 @@
 #include "ephy-sync-utils.h"
 #include "ephy-time-helpers.h"
 #include "ephy-uri-tester-shared.h"
+#include "ephy-web-app-utils.h"
 #include "clear-data-dialog.h"
 #include "cookies-dialog.h"
-#include "languages.h"
+#include "gnome-languages.h"
 #include "passwords-dialog.h"
 #include "synced-tabs-dialog.h"
+#include "webapp-additional-urls-dialog.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <JavaScriptCore/JavaScript.h>
 #include <json-glib/json-glib.h>
+#include <math.h>
 #include <string.h>
 
 #define DOWNLOAD_BUTTON_WIDTH   8
@@ -74,18 +78,26 @@ struct _PrefsDialog {
   GtkWidget *blank_homepage_radiobutton;
   GtkWidget *custom_homepage_radiobutton;
   GtkWidget *custom_homepage_entry;
+  EphyWebApplication *webapp;
+  guint webapp_save_id;
+  GtkWidget *webapp_box;
+  GtkWidget *webapp_icon;
+  GtkWidget *webapp_url;
+  GtkWidget *webapp_title;
   GtkWidget *download_button_hbox;
   GtkWidget *download_button_label;
-  GtkWidget *automatic_downloads_checkbutton;
+  GtkWidget *download_box;
   GtkWidget *search_box;
   GtkWidget *session_box;
+  GtkWidget *browsing_box;
   GtkWidget *restore_session_checkbutton;
   GtkWidget *popups_allow_checkbutton;
   GtkWidget *adblock_allow_checkbutton;
   GtkWidget *enable_plugins_checkbutton;
   GtkWidget *enable_safe_browsing_checkbutton;
+  GtkWidget *enable_smooth_scrolling_checkbutton;
 
-  /* fonts */
+  /* fonts & style */
   GtkWidget *use_gnome_fonts_checkbutton;
   GtkWidget *custom_fonts_table;
   GtkWidget *sans_fontbutton;
@@ -93,6 +105,10 @@ struct _PrefsDialog {
   GtkWidget *mono_fontbutton;
   GtkWidget *css_checkbox;
   GtkWidget *css_edit_button;
+  GtkWidget *default_zoom_spin_button;
+  GtkWidget *reader_mode_box;
+  GtkWidget *reader_mode_font_style;
+  GtkWidget *reader_mode_color_scheme;
 
   /* stored data */
   GtkWidget *always;
@@ -114,9 +130,6 @@ struct _PrefsDialog {
   GtkTreeView *add_lang_treeview;
   GtkTreeModel *lang_model;
 
-  GHashTable *iso_639_table;
-  GHashTable *iso_3166_table;
-
   /* sync */
   GtkWidget *sync_page_box;
   GtkWidget *sync_firefox_iframe_box;
@@ -168,16 +181,17 @@ prefs_dialog_finalize (GObject *object)
     g_object_unref (dialog->add_lang_dialog);
   }
 
-  g_hash_table_destroy (dialog->iso_639_table);
-  g_hash_table_destroy (dialog->iso_3166_table);
-
   if (dialog->fxa_web_view != NULL) {
     webkit_user_content_manager_unregister_script_message_handler (dialog->fxa_manager,
                                                                    "toChromeMessageHandler");
+    webkit_user_content_manager_unregister_script_message_handler (dialog->fxa_manager,
+                                                                   "openWebmailClickHandler");
     webkit_user_script_unref (dialog->fxa_script);
     g_object_unref (dialog->fxa_manager);
   }
 
+  g_clear_pointer (&dialog->webapp, ephy_web_application_free);
+
   G_OBJECT_CLASS (prefs_dialog_parent_class)->finalize (object);
 }
 
@@ -192,7 +206,7 @@ sync_collection_toggled_cb (GtkToggleButton *button,
   if (GTK_WIDGET (button) == dialog->sync_bookmarks_checkbutton) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_bookmarks_manager (shell));
   } else if (GTK_WIDGET (button) == dialog->sync_passwords_checkbutton) {
-    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_password_manager (shell));
+    manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL (shell)));
   } else if (GTK_WIDGET (button) == dialog->sync_history_checkbutton) {
     manager = EPHY_SYNCHRONIZABLE_MANAGER (ephy_shell_get_history_manager (shell));
   } else if (GTK_WIDGET (button) == dialog->sync_open_tabs_checkbutton) {
@@ -464,7 +478,7 @@ sync_message_from_fxa_content_cb (WebKitUserContentManager *manager,
   char *error_msg = NULL;
   gboolean is_error = FALSE;
 
-  message = ephy_embed_utils_get_js_result_as_string (result);
+  message = jsc_value_to_string (webkit_javascript_result_get_js_value (result));
   if (!message) {
     g_warning ("Failed to get JavaScript result as string");
     is_error = TRUE;
@@ -526,6 +540,32 @@ out:
   }
 }
 
+static void
+sync_open_webmail_clicked_cb (WebKitUserContentManager *manager,
+                              WebKitJavascriptResult   *result,
+                              PrefsDialog              *dialog)
+{
+  EphyShell *shell;
+  EphyEmbed *embed;
+  GtkWindow *window;
+  char *url;
+
+  url = jsc_value_to_string (webkit_javascript_result_get_js_value (result));
+  if (url) {
+    /* Open a new tab to the webmail URL. */
+    shell = ephy_shell_get_default ();
+    window = gtk_application_get_active_window (GTK_APPLICATION (shell));
+    embed = ephy_shell_new_tab (shell, EPHY_WINDOW (window),
+                                NULL, EPHY_NEW_TAB_JUMP);
+    ephy_web_view_load_url (ephy_embed_get_web_view (embed), url);
+
+    /* Close the preferences dialog. */
+    gtk_widget_destroy (GTK_WIDGET (dialog));
+
+    g_free (url);
+  }
+}
+
 static void
 sync_setup_firefox_iframe (PrefsDialog *dialog)
 {
@@ -536,11 +576,22 @@ sync_setup_firefox_iframe (PrefsDialog *dialog)
   const char *script;
 
   if (!dialog->fxa_web_view) {
-    script = "function handleToChromeMessage(evt) {"
-             "  let e = JSON.stringify({type: evt.type, detail: evt.detail});"
+    script =
+             /* Handle sign-in messages from the FxA content server. */
+             "function handleToChromeMessage(event) {"
+             "  let e = JSON.stringify({type: event.type, detail: event.detail});"
              "  window.webkit.messageHandlers.toChromeMessageHandler.postMessage(e);"
              "};"
-             "window.addEventListener(\"WebChannelMessageToChrome\", handleToChromeMessage);";
+             "window.addEventListener('WebChannelMessageToChrome', handleToChromeMessage);"
+             /* Handle open-webmail click event. */
+             "function handleOpenWebmailClick(event) {"
+             "  if (event.target.id == 'open-webmail' && event.target.hasAttribute('href'))"
+             "    window.webkit.messageHandlers.openWebmailClickHandler.postMessage(event.target.getAttribute('href'));"
+             "};"
+             "var stage = document.getElementById('stage');"
+             "if (stage)"
+             "  stage.addEventListener('click', handleOpenWebmailClick);";
+
     dialog->fxa_script = webkit_user_script_new (script,
                                                  WEBKIT_USER_CONTENT_INJECT_TOP_FRAME,
                                                  WEBKIT_USER_SCRIPT_INJECT_AT_DOCUMENT_END,
@@ -551,8 +602,14 @@ sync_setup_firefox_iframe (PrefsDialog *dialog)
                       "script-message-received::toChromeMessageHandler",
                       G_CALLBACK (sync_message_from_fxa_content_cb),
                       dialog);
+    g_signal_connect (dialog->fxa_manager,
+                      "script-message-received::openWebmailClickHandler",
+                      G_CALLBACK (sync_open_webmail_clicked_cb),
+                      dialog);
     webkit_user_content_manager_register_script_message_handler (dialog->fxa_manager,
                                                                  "toChromeMessageHandler");
+    webkit_user_content_manager_register_script_message_handler (dialog->fxa_manager,
+                                                                 "openWebmailClickHandler");
 
     shell = ephy_embed_shell_get_default ();
     embed_context = ephy_embed_shell_get_web_context (shell);
@@ -620,7 +677,7 @@ on_sync_synced_tabs_button_clicked (GtkWidget   *button,
   synced_tabs_dialog = synced_tabs_dialog_new (manager);
   gtk_window_set_transient_for (GTK_WINDOW (synced_tabs_dialog), GTK_WINDOW (dialog));
   gtk_window_set_modal (GTK_WINDOW (synced_tabs_dialog), TRUE);
-  gtk_window_present (GTK_WINDOW (synced_tabs_dialog));
+  gtk_window_present_with_time (GTK_WINDOW (synced_tabs_dialog), gtk_get_current_event_time ());
 }
 
 static void
@@ -672,6 +729,165 @@ on_sync_device_name_cancel_button_clicked (GtkWidget   *button,
   g_free (name);
 }
 
+static gboolean
+save_web_application (PrefsDialog *dialog)
+{
+  gboolean changed = FALSE;
+  const char *text;
+
+  dialog->webapp_save_id = 0;
+
+  if (!dialog->webapp)
+    return G_SOURCE_REMOVE;
+
+  text = gtk_entry_get_text (GTK_ENTRY (dialog->webapp_url));
+  if (g_strcmp0 (dialog->webapp->url, text) != 0) {
+    g_free (dialog->webapp->url);
+    dialog->webapp->url = g_strdup (text);
+    changed = TRUE;
+  }
+
+  text = gtk_entry_get_text (GTK_ENTRY (dialog->webapp_title));
+  if (g_strcmp0 (dialog->webapp->name, text) != 0) {
+    g_free (dialog->webapp->name);
+    dialog->webapp->name = g_strdup (text);
+    changed = TRUE;
+  }
+
+  text = (const char *)g_object_get_data (G_OBJECT (dialog->webapp_icon), "ephy-webapp-icon-url");
+  if (g_strcmp0 (dialog->webapp->icon_url, text) != 0) {
+    g_free (dialog->webapp->icon_url);
+    dialog->webapp->icon_url = g_strdup (text);
+    changed = TRUE;
+  }
+
+  if (changed)
+    ephy_web_application_save (dialog->webapp);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+prefs_dialog_save_web_application (PrefsDialog *dialog)
+{
+  if (!dialog->webapp)
+    return;
+
+  if (dialog->webapp_save_id)
+    g_source_remove (dialog->webapp_save_id);
+
+  dialog->webapp_save_id = g_timeout_add_seconds (1, (GSourceFunc)save_web_application, dialog);
+}
+
+static void
+prefs_dialog_update_webapp_icon (PrefsDialog *dialog,
+                                 const char  *icon_url)
+{
+  GdkPixbuf *icon, *scaled_icon;
+  int icon_width, icon_height;
+  double scale;
+
+  icon = gdk_pixbuf_new_from_file (icon_url, NULL);
+  if (!icon)
+    return;
+
+  icon_width = gdk_pixbuf_get_width (icon);
+  icon_height = gdk_pixbuf_get_height (icon);
+  scale = MIN ((double)64 / icon_width, (double)64 / icon_height);
+  scaled_icon = gdk_pixbuf_scale_simple (icon, icon_width * scale, icon_height * scale, GDK_INTERP_NEAREST);
+  g_object_unref (icon);
+  gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->webapp_icon), scaled_icon);
+  g_object_set_data_full (G_OBJECT (dialog->webapp_icon), "ephy-webapp-icon-url",
+                          g_strdup (icon_url), g_free);
+  g_object_unref (scaled_icon);
+}
+
+static void
+webapp_icon_chooser_response_cb (GtkNativeDialog *file_chooser,
+                                 int              response,
+                                 PrefsDialog     *dialog)
+{
+  if (response == GTK_RESPONSE_ACCEPT) {
+    char *icon_url;
+
+    icon_url = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
+    prefs_dialog_update_webapp_icon (dialog, icon_url);
+    g_free (icon_url);
+    prefs_dialog_save_web_application (dialog);
+  }
+
+  g_object_unref (file_chooser);
+}
+
+static void
+on_webapp_icon_button_clicked (GtkWidget   *button,
+                               PrefsDialog *dialog)
+{
+  GtkFileChooser *file_chooser;
+  GSList *pixbuf_formats, *l;
+  GtkFileFilter *images_filter;
+
+  file_chooser = ephy_create_file_chooser (_("Web Application Icon"),
+                                           GTK_WIDGET (dialog),
+                                           GTK_FILE_CHOOSER_ACTION_OPEN,
+                                           EPHY_FILE_FILTER_NONE);
+  images_filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (images_filter, _("Supported Image Files"));
+  gtk_file_chooser_add_filter (file_chooser, images_filter);
+
+  pixbuf_formats = gdk_pixbuf_get_formats ();
+  for (l = pixbuf_formats; l; l = g_slist_next (l)) {
+    GdkPixbufFormat *format = (GdkPixbufFormat *)l->data;
+    GtkFileFilter *filter;
+    gchar *name;
+    gchar **mime_types;
+    guint i;
+
+    if (gdk_pixbuf_format_is_disabled (format) || !gdk_pixbuf_format_is_writable (format))
+      continue;
+
+    filter = gtk_file_filter_new ();
+    name = gdk_pixbuf_format_get_description (format);
+    gtk_file_filter_set_name (filter, name);
+    g_free (name);
+
+    mime_types = gdk_pixbuf_format_get_mime_types (format);
+    for (i = 0; mime_types[i] != 0; i++) {
+      gtk_file_filter_add_mime_type (images_filter, mime_types[i]);
+      gtk_file_filter_add_mime_type (filter, mime_types[i]);
+    }
+    g_strfreev (mime_types);
+
+    gtk_file_chooser_add_filter (file_chooser, filter);
+  }
+  g_slist_free (pixbuf_formats);
+
+  g_signal_connect (file_chooser, "response",
+                    G_CALLBACK (webapp_icon_chooser_response_cb),
+                    dialog);
+
+  gtk_native_dialog_show (GTK_NATIVE_DIALOG (file_chooser));
+}
+
+static void
+on_webapp_entry_changed (GtkEditable *editable,
+                         PrefsDialog *dialog)
+{
+  prefs_dialog_save_web_application (dialog);
+}
+
+static void
+on_manage_webapp_additional_urls_button_clicked (GtkWidget   *button,
+                                                 PrefsDialog *dialog)
+{
+  EphyWebappAdditionalURLsDialog *urls_dialog;
+
+  urls_dialog = ephy_webapp_additional_urls_dialog_new ();
+  gtk_window_set_transient_for (GTK_WINDOW (urls_dialog), GTK_WINDOW (dialog));
+  gtk_window_set_modal (GTK_WINDOW (urls_dialog), TRUE);
+  gtk_window_present_with_time (GTK_WINDOW (urls_dialog), gtk_get_current_event_time ());
+}
+
 static void
 on_manage_cookies_button_clicked (GtkWidget   *button,
                                   PrefsDialog *dialog)
@@ -682,7 +898,7 @@ on_manage_cookies_button_clicked (GtkWidget   *button,
 
   gtk_window_set_transient_for (GTK_WINDOW (cookies_dialog), GTK_WINDOW (dialog));
   gtk_window_set_modal (GTK_WINDOW (cookies_dialog), TRUE);
-  gtk_window_present (GTK_WINDOW (cookies_dialog));
+  gtk_window_present_with_time (GTK_WINDOW (cookies_dialog), gtk_get_current_event_time ());
 }
 
 static void
@@ -692,12 +908,12 @@ on_manage_passwords_button_clicked (GtkWidget   *button,
   EphyPasswordsDialog *passwords_dialog;
   EphyPasswordManager *password_manager;
 
-  password_manager = ephy_shell_get_password_manager (ephy_shell_get_default ());
+  password_manager = ephy_embed_shell_get_password_manager (EPHY_EMBED_SHELL(ephy_shell_get_default ()));
   passwords_dialog = ephy_passwords_dialog_new (password_manager);
 
   gtk_window_set_transient_for (GTK_WINDOW (passwords_dialog), GTK_WINDOW (dialog));
   gtk_window_set_modal (GTK_WINDOW (passwords_dialog), TRUE);
-  gtk_window_present (GTK_WINDOW (passwords_dialog));
+  gtk_window_present_with_time (GTK_WINDOW (passwords_dialog), gtk_get_current_event_time ());
 }
 
 static void
@@ -710,7 +926,36 @@ on_search_engine_dialog_button_clicked (GtkWidget   *button,
 
   gtk_window_set_transient_for (search_engine_dialog, GTK_WINDOW (dialog));
   gtk_window_set_modal (search_engine_dialog, TRUE);
-  gtk_window_present (search_engine_dialog);
+  gtk_window_present_with_time (search_engine_dialog, gtk_get_current_event_time ());
+}
+
+static gboolean
+on_default_zoom_spin_button_output (GtkSpinButton *spin,
+                                    gpointer       user_data)
+{
+  GtkAdjustment *adjustment;
+  g_autofree gchar *text = NULL;
+  gdouble value;
+
+  adjustment = gtk_spin_button_get_adjustment (spin);
+  value = (int)gtk_adjustment_get_value (adjustment);
+  text = g_strdup_printf ("%.f%%", value);
+  gtk_entry_set_text (GTK_ENTRY (spin), text);
+
+  return TRUE;
+}
+
+static void
+on_default_zoom_spin_button_value_changed (GtkSpinButton *spin,
+                                           gpointer       user_data)
+{
+  GtkAdjustment *adjustment;
+  gdouble value;
+
+  adjustment = gtk_spin_button_get_adjustment (spin);
+  value = gtk_adjustment_get_value (adjustment);
+  value = roundf(value) / 100;
+  g_settings_set_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL, value);
 }
 
 static void
@@ -732,18 +977,23 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, blank_homepage_radiobutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, custom_homepage_radiobutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, custom_homepage_entry);
-  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, automatic_downloads_checkbutton);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, webapp_box);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, webapp_icon);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, webapp_url);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, webapp_title);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, search_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, session_box);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, browsing_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, restore_session_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, popups_allow_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, adblock_allow_checkbutton);
-  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, enable_plugins_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, enable_safe_browsing_checkbutton);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, enable_smooth_scrolling_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, download_button_hbox);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, download_button_label);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, download_box);
 
-  /* fonts */
+  /* fonts & style */
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, use_gnome_fonts_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, custom_fonts_table);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sans_fontbutton);
@@ -751,6 +1001,10 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, mono_fontbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, css_checkbox);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, css_edit_button);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, default_zoom_spin_button);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_box);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_font_style);
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, reader_mode_color_scheme);
 
   /* stored data */
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, always);
@@ -793,6 +1047,9 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sync_last_sync_time_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, sync_last_sync_time_label);
 
+  gtk_widget_class_bind_template_callback (widget_class, on_webapp_icon_button_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, on_webapp_entry_changed);
+  gtk_widget_class_bind_template_callback (widget_class, on_manage_webapp_additional_urls_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_manage_cookies_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_manage_passwords_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_search_engine_dialog_button_clicked);
@@ -802,6 +1059,8 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_change_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_save_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_sync_device_name_cancel_button_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, on_default_zoom_spin_button_output);
+  gtk_widget_class_bind_template_callback (widget_class, on_default_zoom_spin_button_value_changed);
 }
 
 static void
@@ -832,8 +1091,12 @@ css_file_created_cb (GObject      *source,
   stream = g_file_create_finish (file, result, &error);
   if (stream == NULL && !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS))
     g_warning ("Failed to create %s: %s", g_file_get_path (file), error->message);
-  else
-    ephy_open_file_via_flatpak_portal (g_file_get_path (file), NULL, css_file_opened_cb, NULL);
+  else {
+    if (ephy_is_running_inside_flatpak ())
+      ephy_open_file_via_flatpak_portal (g_file_get_path (file), NULL, css_file_opened_cb, NULL);
+    else
+      ephy_file_launch_handler (file, gtk_get_current_event_time ());
+  }
 
   if (error != NULL)
     g_error_free (error);
@@ -848,17 +1111,11 @@ css_edit_button_clicked_cb (GtkWidget   *button,
 {
   GFile *css_file;
 
-  css_file = g_file_new_for_path (g_build_filename (ephy_dot_dir (),
+  css_file = g_file_new_for_path (g_build_filename (ephy_profile_dir (),
                                                     USER_STYLESHEET_FILENAME,
                                                     NULL));
 
-  if (ephy_is_running_inside_flatpak ()) {
-    g_file_create_async (css_file, G_FILE_CREATE_NONE, G_PRIORITY_DEFAULT, NULL, css_file_created_cb, NULL);
-  } else {
-    ephy_file_launch_handler ("text/plain", css_file,
-                              gtk_get_current_event_time ());
-    g_object_unref (css_file);
-  }
+  g_file_create_async (css_file, G_FILE_CREATE_NONE, G_PRIORITY_DEFAULT, NULL, css_file_created_cb, NULL);
 }
 
 static void
@@ -1011,50 +1268,6 @@ add_lang_dialog_response_cb (GtkWidget   *widget,
   gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
-static char *
-get_name_for_lang_code (PrefsDialog *pd,
-                        const char  *code)
-{
-  char **str;
-  char *name;
-  const char *langname, *localename;
-  int len;
-
-  str = g_strsplit (code, "-", -1);
-  len = g_strv_length (str);
-  g_assert (len != 0);
-
-  langname = (const char *)g_hash_table_lookup (pd->iso_639_table, str[0]);
-
-  if (len == 1 && langname != NULL) {
-    name = g_strdup (dgettext (ISO_639_DOMAIN, langname));
-  } else if (len == 2 && langname != NULL) {
-    localename = (const char *)g_hash_table_lookup (pd->iso_3166_table, str[1]);
-
-    if (localename != NULL) {
-      /* Translators: the first %s is the language name, and the
-       * second %s is the locale name. Example:
-       * "French (France)"
-       */
-      name = g_strdup_printf (C_("language", "%s (%s)"),
-                              dgettext (ISO_639_DOMAIN, langname),
-                              dgettext (ISO_3166_DOMAIN, localename));
-    } else {
-      name = g_strdup_printf (C_("language", "%s (%s)"),
-                              dgettext (ISO_639_DOMAIN, langname), str[1]);
-    }
-  } else {
-    /* Translators: this refers to a user-define language code
-     * (one which isn't in our built-in list).
-     */
-    name = g_strdup_printf (C_("language", "User defined (%s)"), code);
-  }
-
-  g_strfreev (str);
-
-  return name;
-}
-
 static void
 add_system_language_entry (GtkListStore *store)
 {
@@ -1095,8 +1308,9 @@ setup_add_language_dialog (PrefsDialog *dialog)
   GtkTreeViewColumn *column;
   GtkTreeSelection *selection;
   GtkTreeIter iter;
-  guint i;
+  guint i, n;
   GtkBuilder *builder;
+  g_auto(GStrv) locales;
 
   builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/gtk/prefs-lang-dialog.ui");
   ad = GTK_WIDGET (gtk_builder_get_object (builder, "add_language_dialog"));
@@ -1106,18 +1320,34 @@ setup_add_language_dialog (PrefsDialog *dialog)
 
   store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 
-  for (i = 0; i < G_N_ELEMENTS (languages); i++) {
-    const char *code = languages[i];
-    char *name;
+  locales = gnome_get_all_locales ();
+  n = g_strv_length (locales);
 
-    name = get_name_for_lang_code (dialog, code);
+  for (i = 0; i < n; i++) {
+    const char *locale = locales[i];
+    g_autofree char *language_code = NULL;
+    g_autofree char *country_code = NULL;
+    g_autofree char *language_name = NULL;
+    g_autofree char *shortened_locale = NULL;
+
+    if (!gnome_parse_locale (locale, &language_code, &country_code, NULL, NULL))
+      break;
+
+    if (language_code == NULL)
+      break;
+
+    language_name = gnome_get_language_from_locale (locale, locale);
+
+    if (country_code != NULL)
+      shortened_locale = g_strdup_printf ("%s-%s", language_code, country_code);
+    else
+      shortened_locale = g_strdup (language_code);
 
     gtk_list_store_append (store, &iter);
     gtk_list_store_set (store, &iter,
-                        COL_LANG_NAME, name,
-                        COL_LANG_CODE, code,
+                        COL_LANG_NAME, language_name,
+                        COL_LANG_CODE, shortened_locale,
                         -1);
-    g_free (name);
   }
 
   add_system_language_entry (store);
@@ -1180,7 +1410,7 @@ language_editor_add_button_clicked_cb (GtkWidget   *button,
       (gpointer *)add_lang_dialog);
   }
 
-  gtk_window_present (GTK_WINDOW (pd->add_lang_dialog));
+  gtk_window_present_with_time (GTK_WINDOW (pd->add_lang_dialog), gtk_get_current_event_time ());
 }
 
 static void
@@ -1275,6 +1505,38 @@ language_editor_selection_changed_cb (GtkTreeSelection *selection,
   language_editor_update_buttons (dialog);
 }
 
+static char *
+normalize_locale (const char *locale)
+{
+  char *result = g_strdup (locale);
+
+  /* The result we store in prefs looks like es-ES or en-US. We don't
+   * store codeset (not used in Accept-Langs) and we store with hyphen
+   * instead of underscore (ditto). So here we just uppercase the
+   * country code, converting e.g. es-es to es-ES. We have to do this
+   * because older versions of Epiphany stored locales as entirely
+   * lowercase.
+   */
+  for (char *p = strchr (result, '-'); p != NULL && *p != '\0'; p++)
+    *p = g_ascii_toupper (*p);
+
+  return result;
+}
+
+static char *
+language_for_locale (const char *locale)
+{
+  g_autoptr(GString) string = g_string_new (locale);
+
+  /* Before calling gnome_get_language_from_locale() we have to convert
+   * from web locales (e.g. es-ES) to UNIX (e.g. es_ES.UTF-8).
+   */
+  g_strdelimit (string->str, "-", '_');
+  g_string_append (string, ".UTF-8");
+
+  return gnome_get_language_from_locale (string->str, string->str);
+}
+
 static void
 create_language_section (PrefsDialog *dialog)
 {
@@ -1286,9 +1548,6 @@ create_language_section (PrefsDialog *dialog)
   char **list = NULL;
   int i;
 
-  dialog->iso_639_table = ephy_langs_iso_639_table ();
-  dialog->iso_3166_table = ephy_langs_iso_3166_table ();
-
   g_signal_connect (dialog->lang_add_button, "clicked",
                     G_CALLBACK (language_editor_add_button_clicked_cb), dialog);
   g_signal_connect (dialog->lang_remove_button, "clicked",
@@ -1334,16 +1593,17 @@ create_language_section (PrefsDialog *dialog)
 
   /* Fill languages editor */
   for (i = 0; list[i]; i++) {
-    const char *code = (const char *)list[i];
-
+    const char *code = list[i];
     if (strcmp (code, "system") == 0) {
       add_system_language_entry (store);
     } else if (code[0] != '\0') {
-      char *text;
-
-      text = get_name_for_lang_code (dialog, code);
-      language_editor_add (dialog, code, text);
-      g_free (text);
+      g_autofree char *normalized_locale = normalize_locale (code);
+      if (normalized_locale != NULL) {
+        g_autofree char *language_name = language_for_locale (normalized_locale);
+        if (language_name == NULL)
+          language_name = g_strdup (normalized_locale);
+        language_editor_add (dialog, normalized_locale, language_name);
+      }
     }
   }
   g_object_unref (store);
@@ -1409,11 +1669,16 @@ create_download_path_button (PrefsDialog *dialog)
 }
 
 static void
-prefs_dialog_response_cb (GtkDialog *widget,
-                          int        response,
-                          GtkDialog *dialog)
+prefs_dialog_response_cb (GtkWidget   *widget,
+                          int          response,
+                          PrefsDialog *dialog)
 {
-  gtk_widget_destroy (GTK_WIDGET (dialog));
+  if (dialog->webapp_save_id) {
+    g_source_remove (dialog->webapp_save_id);
+    save_web_application (dialog);
+  }
+
+  gtk_widget_destroy (widget);
 }
 
 static void
@@ -1445,7 +1710,7 @@ clear_personal_data_button_clicked_cb (GtkWidget   *button,
                                NULL);
   gtk_window_set_transient_for (GTK_WINDOW (clear_dialog), GTK_WINDOW (dialog));
   gtk_window_set_modal (GTK_WINDOW (clear_dialog), TRUE);
-  gtk_window_present (GTK_WINDOW (clear_dialog));
+  gtk_window_present_with_time (GTK_WINDOW (clear_dialog), gtk_get_current_event_time ());
 }
 
 static gboolean
@@ -1661,6 +1926,13 @@ setup_general_page (PrefsDialog *dialog)
   GSettings *settings;
   GSettings *web_settings;
 
+  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    dialog->webapp = ephy_web_application_for_profile_directory (ephy_profile_dir ());
+    prefs_dialog_update_webapp_icon (dialog, dialog->webapp->icon_url);
+    gtk_entry_set_text (GTK_ENTRY (dialog->webapp_url), dialog->webapp->url);
+    gtk_entry_set_text (GTK_ENTRY (dialog->webapp_title), dialog->webapp->name);
+  }
+
   settings = ephy_settings_get (EPHY_PREFS_SCHEMA);
   web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
 
@@ -1707,11 +1979,6 @@ setup_general_page (PrefsDialog *dialog)
                     G_CALLBACK (custom_homepage_entry_icon_released),
                     NULL);
 
-  g_settings_bind (settings,
-                   EPHY_PREFS_AUTO_DOWNLOADS,
-                   dialog->automatic_downloads_checkbutton,
-                   "active",
-                   G_SETTINGS_BIND_DEFAULT);
   g_settings_bind_with_mapping (settings,
                                 EPHY_PREFS_RESTORE_SESSION_POLICY,
                                 dialog->restore_session_checkbutton,
@@ -1725,11 +1992,6 @@ setup_general_page (PrefsDialog *dialog)
                    dialog->popups_allow_checkbutton,
                    "active",
                    G_SETTINGS_BIND_INVERT_BOOLEAN);
-  g_settings_bind (web_settings,
-                   EPHY_PREFS_WEB_ENABLE_PLUGINS,
-                   dialog->enable_plugins_checkbutton,
-                   "active",
-                   G_SETTINGS_BIND_DEFAULT);
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_ENABLE_SAFE_BROWSING,
                    dialog->enable_safe_browsing_checkbutton,
@@ -1757,19 +2019,136 @@ setup_general_page (PrefsDialog *dialog)
                     "clicked",
                     G_CALLBACK (do_not_track_button_clicked_cb),
                     dialog);
+  g_settings_bind (web_settings,
+                   EPHY_PREFS_WEB_ENABLE_SMOOTH_SCROLLING,
+                   dialog->enable_smooth_scrolling_checkbutton,
+                   "active",
+                   G_SETTINGS_BIND_DEFAULT);
 
   if (ephy_is_running_inside_flatpak ())
-    gtk_widget_hide (dialog->download_button_label);
+    gtk_widget_hide (dialog->download_box);
   else
     create_download_path_button (dialog);
 }
 
+enum {
+  ENUM_MODEL_COLUMN_LABEL,
+  ENUM_MODEL_COLUMN_NICK,
+  ENUM_MODEL_N_COLUMNS,
+};
+
+typedef struct {
+  gint        value;
+  const char *label;
+} EnumModelItem;
+
+static GtkTreeModel *
+enum_model_new (GType                enum_type,
+                gsize                n_items,
+                const EnumModelItem *items)
+{
+  GtkListStore *model;
+  GEnumClass *enum_class;
+  gsize i;
+
+  enum_class = g_type_class_ref (enum_type);
+  model = gtk_list_store_new (ENUM_MODEL_N_COLUMNS,
+                              G_TYPE_STRING,
+                              G_TYPE_STRING);
+
+  for (i = 0; i < n_items; i++) {
+    GtkTreeIter iter;
+    const char *nick;
+    guint  j;
+
+    nick = NULL;
+    for (j = 0; j < enum_class->n_values; j++) {
+      if (enum_class->values[j].value == items[i].value) {
+        nick = enum_class->values[j].value_nick;
+        break;
+      }
+    }
+    g_assert (nick != NULL);
+
+    gtk_list_store_append (model, &iter);
+    gtk_list_store_set (model,
+                        &iter,
+                        ENUM_MODEL_COLUMN_LABEL, _(items[i].label),
+                        ENUM_MODEL_COLUMN_NICK, nick,
+                        -1);
+  }
+
+  g_type_class_unref (enum_class);
+  return GTK_TREE_MODEL (model);
+}
+
+static void
+enum_model_attach (GtkTreeModel *model,
+                   GtkComboBox  *combobox,
+                   GSettings    *settings,
+                   const char   *key)
+{
+  GtkCellRenderer *renderer;
+
+  gtk_cell_layout_clear (GTK_CELL_LAYOUT (combobox));
+
+  gtk_combo_box_set_model (combobox, model);
+  gtk_combo_box_set_id_column (combobox,
+                               ENUM_MODEL_COLUMN_NICK);
+
+  g_settings_bind (settings,
+                   key,
+                   combobox,
+                   "active-id",
+                   G_SETTINGS_BIND_DEFAULT);
+
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox),
+                              renderer,
+                              FALSE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combobox),
+                                 renderer,
+                                 "text",
+                                 ENUM_MODEL_COLUMN_LABEL);
+}
+
+static const EnumModelItem reader_font_style_items[] = {
+  { EPHY_PREFS_READER_FONT_STYLE_SANS,  N_("Sans")  },
+  { EPHY_PREFS_READER_FONT_STYLE_SERIF, N_("Serif") },
+};
+
+static const EnumModelItem reader_color_scheme_items[] = {
+  { EPHY_PREFS_READER_COLORS_LIGHT, N_("Light") },
+  { EPHY_PREFS_READER_COLORS_DARK,  N_("Dark")  },
+};
+
 static void
 setup_fonts_page (PrefsDialog *dialog)
 {
   GSettings *web_settings;
+  GSettings *reader_settings;
+  GtkTreeModel *model;
 
   web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
+  reader_settings = ephy_settings_get (EPHY_PREFS_READER_SCHEMA);
+
+  model = enum_model_new (EPHY_TYPE_PREFS_READER_FONT_STYLE,
+                          G_N_ELEMENTS (reader_font_style_items),
+                          reader_font_style_items);
+  enum_model_attach (model,
+                     GTK_COMBO_BOX (dialog->reader_mode_font_style),
+                     reader_settings,
+                     EPHY_PREFS_READER_FONT_STYLE);
+  g_object_unref (model);
+
+  model = enum_model_new (EPHY_TYPE_PREFS_READER_COLOR_SCHEME,
+                          G_N_ELEMENTS (reader_color_scheme_items),
+                          reader_color_scheme_items);
+  enum_model_attach (model,
+                     GTK_COMBO_BOX (dialog->reader_mode_color_scheme),
+                     reader_settings,
+                     EPHY_PREFS_READER_COLOR_SCHEME);
+  g_object_unref (model);
 
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_USE_GNOME_FONTS,
@@ -1807,10 +2186,14 @@ setup_fonts_page (PrefsDialog *dialog)
                    dialog->css_edit_button,
                    "sensitive",
                    G_SETTINGS_BIND_GET);
+
   g_signal_connect (dialog->css_edit_button,
                     "clicked",
                     G_CALLBACK (css_edit_button_clicked_cb),
                     dialog);
+
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->default_zoom_spin_button),
+                             g_settings_get_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL) * 100);
 }
 
 static void
@@ -2000,16 +2383,22 @@ prefs_dialog_init (PrefsDialog *dialog)
   gtk_widget_init_template (GTK_WIDGET (dialog));
 
   mode = ephy_embed_shell_get_mode (ephy_embed_shell_get_default ());
+  gtk_widget_set_visible (dialog->webapp_box,
+                          mode == EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (dialog->homepage_box,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (dialog->search_box,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
-  gtk_widget_set_visible (dialog->automatic_downloads_checkbutton,
-                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (dialog->session_box,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
+  gtk_widget_set_visible (dialog->browsing_box,
+                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
   gtk_widget_set_visible (dialog->do_not_track_checkbutton,
                           mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
+  gtk_widget_set_visible (dialog->enable_smooth_scrolling_checkbutton,
+                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
+  gtk_widget_set_visible (dialog->reader_mode_box,
+                          mode != EPHY_EMBED_SHELL_MODE_APPLICATION);
 
   setup_general_page (dialog);
   setup_fonts_page (dialog);
