@@ -109,41 +109,6 @@ profile_dir_exists (void)
   return FALSE;
 }
 
-static void
-migrate_app_desktop_file_categories (void)
-{
-  GList *web_apps, *l;
-
-  web_apps = ephy_web_application_get_legacy_application_list ();
-
-  for (l = web_apps; l; l = l->next) {
-    EphyWebApplication *app = (EphyWebApplication *)l->data;
-    GKeyFile *file;
-    char *data = NULL;
-    char *app_path;
-    char *desktop_file_path;
-
-    file = g_key_file_new ();
-
-    app_path = ephy_web_application_get_profile_directory (app->id);
-    desktop_file_path = g_build_filename (app_path, app->desktop_file, NULL);
-    g_key_file_load_from_file (file, desktop_file_path, G_KEY_FILE_NONE, NULL);
-
-    LOG ("migrate_app_desktop_file_categories: adding Categories to %s", app->name);
-    g_key_file_set_value (file, "Desktop Entry", "Categories", "Network;GNOME;GTK;");
-
-    data = g_key_file_to_data (file, NULL, NULL);
-    g_file_set_contents (desktop_file_path, data, -1, NULL);
-
-    g_free (app_path);
-    g_free (desktop_file_path);
-    g_free (data);
-    g_key_file_free (file);
-  }
-
-  ephy_web_application_free_application_list (web_apps);
-}
-
 /* https://bugzilla.gnome.org/show_bug.cgi?id=752738 */
 static void
 migrate_insecure_password (SecretItem *item)
@@ -1044,16 +1009,20 @@ migrate_annoyance_list (void)
 }
 
 static gboolean
-move_directory_contents (GFile *source,
-                         GFile *dest)
+move_directory_contents (const char *source_path,
+                         const char *dest_path)
 {
+  g_autoptr(GFile) source = g_file_new_for_path (source_path);
+  g_autoptr(GFile) dest = g_file_new_for_path (dest_path);
+  g_autoptr(GFileEnumerator) direnum = NULL;
+  g_autoptr(GError) error = NULL;
+
   /* Just a sanity check as it should already exist */
   g_file_make_directory (dest, NULL, NULL);
 
-  g_autoptr(GError) error = NULL;
-  g_autoptr(GFileEnumerator) direnum = g_file_enumerate_children (source, G_FILE_ATTRIBUTE_STANDARD_NAME,
-                                                                  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-                                                                  NULL, &error);
+  direnum = g_file_enumerate_children (source, G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                       G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                       NULL, &error);
   if (error) {
     g_warning ("Failed to enumerate files: %s", error->message);
     return FALSE;
@@ -1088,7 +1057,10 @@ migrate_profile_directories (void)
 {
   GList *web_apps, *l;
 
-  /* Web app profiles moved from config to data dirs */
+  /* Web app profiles moved from config to data dirs. If this migration has
+   * already been run for a separate profile, then the legacy application list
+   * should be empty, so it's harmless to run this multiple times.
+   */
   web_apps = ephy_web_application_get_legacy_application_list ();
   for (l = web_apps; l; l = l->next) {
     EphyWebApplication *app = (EphyWebApplication *)l->data;
@@ -1096,14 +1068,10 @@ migrate_profile_directories (void)
 
     g_autofree char *old_name = g_strconcat ("app-epiphany-", app->id, NULL);
     g_autofree char *old_path = g_build_filename (legacy_profile_dir (), old_name, NULL);
-    g_autoptr(GFile) old_directory = g_file_new_for_path (old_path);
-
     g_autofree char *app_path = ephy_web_application_get_profile_directory (app->id);
-    g_autoptr(GFile) new_directory = g_file_new_for_path (app_path);
 
-    if (!move_directory_contents (old_directory, new_directory)) {
+    if (!move_directory_contents (old_path, app_path))
       continue;
-    }
 
     // Create an empty file to indicate it's an app
     g_autofree char *app_file = g_build_filename (app_path, ".app", NULL);
@@ -1160,14 +1128,17 @@ migrate_profile_directories (void)
 
   ephy_web_application_free_application_list (web_apps);
 
-  /* The default profile also changed directories */
-  g_autoptr(GFile) old_directory = g_file_new_for_path (legacy_default_profile_dir ());
-  g_autoptr(GFile) new_directory = g_file_new_for_path (ephy_default_profile_dir ());
+  /* The default profile also changed directories. This needs to run only once,
+   * for the main browser instance.
+   */
+  if (!ephy_profile_dir_is_default ())
+    return;
 
-  if (!move_directory_contents (old_directory, new_directory))
+  if (!move_directory_contents (legacy_default_profile_dir (), ephy_default_profile_dir ()))
     return;
 
   /* We are also moving some cache directories so just remove the old ones */
+  g_autoptr(GFile) new_directory = g_file_new_for_path (ephy_default_profile_dir ());
   g_autoptr(GFile) adblock_directory = g_file_get_child (new_directory, "adblock");
   g_file_delete (adblock_directory, NULL, NULL);
   g_autoptr(GFile) gsb_file = g_file_get_child (new_directory, "gsb-threats.db");
@@ -1176,6 +1147,100 @@ migrate_profile_directories (void)
   g_file_delete (gsb_journal_file, NULL, NULL);
 }
 
+static void
+migrate_web_extension_config_dir (void)
+{
+  /* Epiphany 3.32.0 web process created its config dir in the wrong place by
+   * mistake for default profile dirs. It only contains read-only data copied
+   * from dconf, so we can just delete it.
+   */
+  g_autofree char *path = g_build_filename (ephy_default_profile_dir (), "config", NULL);
+  ephy_file_delete_dir_recursively (path, NULL);
+}
+
+static void
+migrate_webapps_harder (void)
+{
+  /* We created some webapps in the default profile directory by mistake.
+   * Whoopsie!
+   */
+  g_autoptr(GFileEnumerator) children = NULL;
+  g_autoptr(GFileInfo) info;
+  g_autofree char *parent_directory_path = NULL;
+  g_autoptr(GFile) parent_directory = NULL;
+
+  parent_directory_path = g_build_filename (g_get_user_data_dir (), "epiphany", NULL);
+
+  parent_directory = g_file_new_for_path (parent_directory_path);
+  children = g_file_enumerate_children (parent_directory,
+                                        "standard::name",
+                                        0, NULL, NULL);
+  if (!children)
+    return;
+
+  info = g_file_enumerator_next_file (children, NULL, NULL);
+  while (info) {
+    const char *name = g_file_info_get_name (info);
+
+    if (g_str_has_prefix (name, "epiphany-")) {
+      g_autofree char *incorrect_profile_dir = g_build_filename (parent_directory_path, name, NULL);
+      g_autofree char *correct_profile_dir = g_build_filename (g_get_user_data_dir (), name, NULL);
+      g_autofree char *app_file = g_build_filename (incorrect_profile_dir, ".app", NULL);
+
+      if (g_file_test (app_file, G_FILE_TEST_EXISTS)) {
+        g_autoptr(GKeyFile) file = g_key_file_new ();
+        g_autofree char *desktop_file_name = g_strconcat (name, ".desktop", NULL);
+        g_autofree char *desktop_file_path = g_build_filename (correct_profile_dir, desktop_file_name, NULL);
+        g_autoptr(GError) error = NULL;
+
+        move_directory_contents (incorrect_profile_dir, correct_profile_dir);
+
+        // Update Exec and Icon to point to the new profile dir
+        g_key_file_load_from_file (file, desktop_file_path, G_KEY_FILE_NONE, NULL);
+
+        g_autofree char *exec = g_key_file_get_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_EXEC, &error);
+        if (exec == NULL) {
+          g_warning ("Failed to get Exec key from %s: %s", desktop_file_path, error->message);
+          continue;
+        }
+        g_autofree char *new_exec = ephy_string_find_and_replace (exec, incorrect_profile_dir, correct_profile_dir);
+        LOG ("migrate_profile_directories: setting Exec to %s", new_exec);
+        g_key_file_set_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_EXEC, new_exec);
+
+        g_autofree char *icon = g_key_file_get_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, &error);
+        if (exec == NULL) {
+          g_warning ("Failed to get Icon key from %s: %s", desktop_file_path, error->message);
+          continue;
+        }
+        g_autofree char *new_icon = ephy_string_find_and_replace (icon, incorrect_profile_dir, correct_profile_dir);
+        LOG ("migrate_profile_directories: setting Icon to %s", new_icon);
+        g_key_file_set_string (file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_ICON, new_icon);
+
+        if (!g_key_file_save_to_file (file, desktop_file_path, &error)) {
+          g_warning ("Failed to save desktop file %s", error->message);
+          g_clear_error (&error);
+        }
+
+        g_autofree char *desktop_symlink_path = g_build_filename (g_get_user_data_dir (), "applications", desktop_file_name, NULL);
+        g_autoptr(GFile) desktop_symlink = g_file_new_for_path (desktop_symlink_path);
+        LOG ("Symlinking %s to %s", desktop_symlink_path, desktop_file_path);
+
+        // Try removing old symlink, failure is ok assuming it doesn't exist.
+        if (!g_file_delete (desktop_symlink, NULL, &error)) {
+          g_warning ("Failed to remove old symbolic link: %s", error->message);
+          g_clear_error (&error);
+        }
+
+        if (!g_file_make_symbolic_link (desktop_symlink, desktop_file_path, NULL, &error))
+          g_warning ("Failed to make symbolic link: %s", error->message);
+      }
+    }
+
+    g_clear_object (&info);
+    info = g_file_enumerator_next_file (children, NULL, NULL);
+  }
+}
+
 static void
 migrate_nothing (void)
 {
@@ -1277,7 +1342,7 @@ const EphyProfileMigrator migrators[] = {
   /*  7 */ migrate_nothing,
   /*  8 */ migrate_nothing,
   /*  9 */ migrate_nothing,
-  /* 10 */ migrate_app_desktop_file_categories,
+  /* 10 */ migrate_nothing,
   /* 11 */ migrate_insecure_passwords,
   /* 12 */ migrate_bookmarks,
   /* 13 */ migrate_adblock_filters,
@@ -1298,6 +1363,8 @@ const EphyProfileMigrator migrators[] = {
   /* 28 */ migrate_annoyance_list,
   /* 29 */ migrate_zoom_level,
   /* 30 */ migrate_profile_directories,
+  /* 31 */ migrate_web_extension_config_dir,
+  /* 32 */ migrate_webapps_harder,
 };
 
 static gboolean
@@ -1305,6 +1372,7 @@ ephy_migrator (void)
 {
   int latest, i;
   EphyProfileMigrator m;
+  g_autofree char *legacy_migration_file = NULL;
 
   /* If after this point there's no profile dir, there's no point in
    * running anything because Epiphany has never run in this system, so
@@ -1328,8 +1396,11 @@ ephy_migrator (void)
     return TRUE;
   }
 
-  latest = MAX (ephy_profile_utils_get_migration_version (),
-                ephy_profile_utils_get_migration_version_for_profile_dir (legacy_profile_dir ()));
+  legacy_migration_file = g_build_filename (legacy_profile_dir (), ".migrated", NULL);
+  if (g_file_test (legacy_migration_file, G_FILE_TEST_EXISTS))
+    latest = ephy_profile_utils_get_migration_version_for_profile_dir (legacy_profile_dir ());
+  else
+    latest = ephy_profile_utils_get_migration_version ();
 
   LOG ("Running migrators up to version %d, current migration version is %d.",
        EPHY_PROFILE_MIGRATION_VERSION, latest);
