@@ -53,6 +53,7 @@
 #include "ephy-shell.h"
 #include "ephy-string.h"
 #include "ephy-vcs-version.h"
+#include "ephy-view-source-handler.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-zoom.h"
 
@@ -389,14 +390,14 @@ window_cmd_import_bookmarks (GSimpleAction *action,
   GtkTreeModel *tree_model;
   GtkCellRenderer *cell_renderer;
 
-  dialog = gtk_dialog_new_with_buttons (_("Import Bookmarks"),
-                                        GTK_WINDOW (window),
-                                        GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_USE_HEADER_BAR,
-                                        _("_Cancel"),
-                                        GTK_RESPONSE_CANCEL,
-                                        _("Ch_oose File"),
-                                        GTK_RESPONSE_OK,
-                                        NULL);
+  dialog = hdy_dialog_new (GTK_WINDOW (window));
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Import Bookmarks"));
+  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                          _("_Cancel"),
+                          GTK_RESPONSE_CANCEL,
+                          _("Ch_oose File"),
+                          GTK_RESPONSE_OK,
+                          NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
   content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
@@ -498,7 +499,7 @@ window_cmd_show_history (GSimpleAction *action,
   if (GTK_WINDOW (user_data) != gtk_window_get_transient_for (GTK_WINDOW (dialog)))
     gtk_window_set_transient_for (GTK_WINDOW (dialog),
                                   GTK_WINDOW (user_data));
-  gtk_window_present (GTK_WINDOW (dialog));
+  gtk_window_present_with_time (GTK_WINDOW (dialog), gtk_get_current_event_time ());
 }
 
 void
@@ -514,7 +515,7 @@ window_cmd_show_preferences (GSimpleAction *action,
     gtk_window_set_transient_for (dialog,
                                   GTK_WINDOW (user_data));
 
-  gtk_window_present (dialog);
+  gtk_window_present_with_time (dialog, gtk_get_current_event_time ());
 }
 
 void
@@ -541,7 +542,7 @@ window_cmd_show_shortcuts (GSimpleAction *action,
   if (gtk_window_get_transient_for (GTK_WINDOW (shortcuts_window)) != GTK_WINDOW (user_data))
     gtk_window_set_transient_for (GTK_WINDOW (shortcuts_window), GTK_WINDOW (user_data));
 
-  gtk_window_present (GTK_WINDOW (shortcuts_window));
+  gtk_window_present_with_time (GTK_WINDOW (shortcuts_window), gtk_get_current_event_time ());
 }
 
 void
@@ -560,84 +561,29 @@ window_cmd_show_about (GSimpleAction *action,
                        gpointer       user_data)
 {
   EphyWindow *window = EPHY_WINDOW (user_data);
+  GtkAboutDialog *dialog;
   char *comments = NULL;
   GKeyFile *key_file;
   GBytes *bytes;
   GError *error = NULL;
-  char **list, **authors, **maintainers, **past_maintainers, **contributors, **artists, **documenters;
-  gsize n_authors, n_maintainers, n_past_maintainers, n_contributors, n_artists, n_documenters, i, j;
+  char **authors, **maintainers, **past_maintainers, **contributors, **artists, **documenters;
 
   key_file = g_key_file_new ();
   bytes = g_resources_lookup_data ("/org/gnome/epiphany/about.ini", 0, NULL);
   if (!g_key_file_load_from_data (key_file, g_bytes_get_data (bytes, NULL), -1, 0, &error)) {
     g_warning ("Couldn't load about data: %s\n", error->message);
     g_error_free (error);
+    g_key_file_free (key_file);
     return;
   }
   g_bytes_unref (bytes);
 
-  list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Author",
-                                     &n_authors, NULL);
-  maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Maintainers",
-                                            &n_maintainers, NULL);
-  past_maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "PastMaintainers",
-                                                 &n_past_maintainers, NULL);
-  contributors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Contributors",
-                                             &n_contributors, NULL);
-
-#define APPEND(_to, _from) \
-  _to[i++] = g_strdup (_from);
-
-#define APPEND_STRV_AND_FREE(_to, _from) \
-  if (_from) \
-  { \
-    for (j = 0; _from[j] != NULL; ++j) \
-    { \
-      _to[i++] = _from[j]; \
-    } \
-    g_free (_from); \
-  }
-
-  authors = g_new (char *, (list ? n_authors : 0) +
-                   (maintainers ? n_maintainers : 0) +
-                   (past_maintainers ? n_past_maintainers : 0) +
-                   (contributors ? n_contributors : 0) + 9 + 1);
-  i = 0;
-  APPEND_STRV_AND_FREE (authors, list);
-  APPEND (authors, "");
-  APPEND (authors, _("Current maintainers:"));
-  APPEND_STRV_AND_FREE (authors, maintainers);
-  APPEND (authors, "");
-  APPEND (authors, _("Contact us at:"));
-  APPEND (authors, "<epiphany-list@gnome.org>");
-  APPEND (authors, "");
-  APPEND (authors, _("Past maintainers:"));
-  APPEND_STRV_AND_FREE (authors, past_maintainers);
-  APPEND (authors, "");
-  APPEND (authors, _("Contributors:"));
-  APPEND_STRV_AND_FREE (authors, contributors);
-  authors[i++] = NULL;
-
-  list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Artists", &n_artists, NULL);
-
-  artists = g_new (char *, (list ? n_artists : 0) + 4 + 1);
-  i = 0;
-  APPEND_STRV_AND_FREE (artists, list);
-  artists[i++] = NULL;
-
-  list = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Documenters", &n_documenters, NULL);
-
-  documenters = g_new (char *, (list ? n_documenters : 0) + 3 + 1);
-  i = 0;
-  APPEND_STRV_AND_FREE (documenters, list);
-  APPEND (documenters, "");
-  APPEND (documenters, _("Contact us at:"));
-  APPEND (documenters, "<gnome-doc-list@gnome.org>");
-  documenters[i++] = NULL;
-
-#undef APPEND
-#undef APPEND_STRV_AND_FREE
-
+  authors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Author", NULL, NULL);
+  maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Maintainers", NULL, NULL);
+  past_maintainers = g_key_file_get_string_list (key_file, ABOUT_GROUP, "PastMaintainers", NULL, NULL);
+  contributors = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Contributors", NULL, NULL);
+  artists = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Artists", NULL, NULL);
+  documenters = g_key_file_get_string_list (key_file, ABOUT_GROUP, "Documenters", NULL, NULL);
   g_key_file_free (key_file);
 
   comments = g_strdup_printf (_("A simple, clean, beautiful view of the web.\n"
@@ -646,43 +592,46 @@ window_cmd_show_about (GSimpleAction *action,
                               webkit_get_minor_version (),
                               webkit_get_micro_version ());
 
-  gtk_show_about_dialog (window ? GTK_WINDOW (window) : NULL,
+  dialog = GTK_ABOUT_DIALOG (gtk_about_dialog_new ());
+
+  if (window)
+    gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
+
 #if !TECH_PREVIEW
-                         "program-name", _("Web"),
+  gtk_about_dialog_set_program_name (dialog, _("Web"));
 #else
-                         "program-name", _("Epiphany Technology Preview"),
+  gtk_about_dialog_set_program_name (dialog, _("Epiphany Technology Preview"));
 #endif
-                         "version", VCSVERSION,
-                         "copyright", "Copyright © 2002–2004 Marco Pesenti Gritti\n"
-                         "Copyright © 2003–2018 The Web Developers",
-                         "artists", artists,
-                         "authors", authors,
-                         "comments", comments,
-                         "documenters", documenters,
-                         /* Translators: This is a special message that shouldn't be translated
-                          * literally. It is used in the about box to give credits to
-                          * the translators.
-                          * Thus, you should translate it to your name and email address.
-                          * You should also include other translators who have contributed to
-                          * this translation; in that case, please write each of them on a separate
-                          * line seperated by newlines (\n).
-                          */
-                         "translator-credits", _("translator-credits"),
-                         "logo-icon-name", "org.gnome.Epiphany",
-                         "website", "https://wiki.gnome.org/Apps/Web",
-#if !TECH_PREVIEW
-                         "website-label", _("Web Website"),
-#else
-                         "website-label", _("Website"),
-#endif
-                         "license-type", GTK_LICENSE_GPL_3_0,
-                         "wrap-license", TRUE,
-                         NULL);
+
+  gtk_about_dialog_set_version (dialog, VCSVERSION);
+  gtk_about_dialog_set_copyright (dialog, "Copyright © 2002–2004 Marco Pesenti Gritti\n"
+                                          "Copyright © 2003–2019 The GNOME Web Developers");
+  gtk_about_dialog_set_comments (dialog, comments);
+  gtk_about_dialog_set_license_type (dialog, GTK_LICENSE_GPL_3_0);
+  gtk_about_dialog_set_website (dialog, "https://wiki.gnome.org/Apps/Web");
+  gtk_about_dialog_set_website_label (dialog, _("Website"));
+  gtk_about_dialog_set_logo_icon_name (dialog, "org.gnome.Epiphany");
+
+  /* We have to use add_credit_section() for documenters and artists to
+   * ensure maintainers get sorted first. */
+  gtk_about_dialog_set_authors (dialog, (const char **)authors);
+  gtk_about_dialog_set_translator_credits (dialog, _("translator-credits"));
+  gtk_about_dialog_add_credit_section (dialog, _("Current maintainers"), (const char **)maintainers);
+  gtk_about_dialog_add_credit_section (dialog, _("Past maintainers"), (const char **)past_maintainers);
+  gtk_about_dialog_add_credit_section (dialog, _("Documented by"), (const char **)documenters);
+  gtk_about_dialog_add_credit_section (dialog, _("Artwork by"), (const char **)artists);
+  gtk_about_dialog_add_credit_section (dialog, _("Contributors"), (const char **)contributors);
+
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (GTK_WIDGET (dialog));
 
   g_free (comments);
   g_strfreev (artists);
   g_strfreev (authors);
+  g_strfreev (contributors);
   g_strfreev (documenters);
+  g_strfreev (maintainers);
+  g_strfreev (past_maintainers);
 }
 
 void
@@ -797,29 +746,6 @@ window_cmd_stop (GSimpleAction *action,
   webkit_web_view_stop_loading (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 }
 
-static gboolean
-event_with_shift (void)
-{
-  GdkEvent *event;
-  GdkEventType type = 0;
-  guint state = 0;
-
-  event = gtk_get_current_event ();
-  if (event) {
-    type = event->type;
-
-    if (type == GDK_BUTTON_RELEASE) {
-      state = event->button.state;
-    } else if (type == GDK_KEY_PRESS || type == GDK_KEY_RELEASE) {
-      state = event->key.state;
-    }
-
-    gdk_event_free (event);
-  }
-
-  return (state & GDK_SHIFT_MASK) != 0;
-}
-
 void
 window_cmd_reload (GSimpleAction *action,
                    GVariant      *parameter,
@@ -829,17 +755,31 @@ window_cmd_reload (GSimpleAction *action,
   EphyEmbed *embed;
   WebKitWebView *view;
 
-  embed = ephy_embed_container_get_active_child
-            (EPHY_EMBED_CONTAINER (window));
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_assert (embed != NULL);
 
   gtk_widget_grab_focus (GTK_WIDGET (embed));
 
   view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
-  if (event_with_shift ())
-    webkit_web_view_reload_bypass_cache (view);
-  else
-    webkit_web_view_reload (view);
+  webkit_web_view_reload (view);
+}
+
+void
+window_cmd_reload_bypass_cache (GSimpleAction *action,
+                                GVariant      *parameter,
+                                gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  WebKitWebView *view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+
+  view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+  webkit_web_view_reload_bypass_cache (view);
 }
 
 void window_cmd_combined_stop_reload (GSimpleAction *action,
@@ -934,28 +874,6 @@ window_cmd_open (GSimpleAction *action,
   gtk_native_dialog_show (GTK_NATIVE_DIALOG (dialog));
 }
 
-void
-window_cmd_open_in_browser (GSimpleAction *action,
-                            GVariant      *parameter,
-                            gpointer       user_data)
-{
-  EphyWindow *window = user_data;
-  EphyEmbed *embed;
-  EphyWebView *view;
-  const char *uri;
-
-  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-  g_assert (embed != NULL);
-
-  view = ephy_embed_get_web_view (embed);
-  uri = ephy_web_view_get_address (view);
-
-  ephy_file_open_uri_in_default_browser (uri, GDK_CURRENT_TIME,
-                                         gtk_window_get_screen (GTK_WINDOW (window)));
-
-  ephy_window_close_active_child (window);
-}
-
 typedef struct {
   EphyWebView *view;
   GtkWidget *image;
@@ -970,7 +888,7 @@ static void
 ephy_application_dialog_data_free (EphyApplicationDialogData *data)
 {
   g_free (data->icon_href);
-  g_slice_free (EphyApplicationDialogData, data);
+  g_free (data);
 }
 
 static void
@@ -1248,14 +1166,15 @@ set_default_application_title (EphyApplicationDialogData *data,
     uri = soup_uri_new (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)));
     host = soup_uri_get_host (uri);
 
-    if (host != NULL && host[0] != '\0')
+    if (host != NULL && host[0] != '\0') {
       title = get_special_case_application_title_for_host (host);
 
-    if (title == NULL || title[0] == '\0') {
-      if (g_str_has_prefix (host, "www."))
-        title = g_strdup (host + strlen ("www."));
-      else
-        title = g_strdup (host);
+      if (title == NULL || title[0] == '\0') {
+        if (g_str_has_prefix (host, "www."))
+          title = g_strdup (host + strlen ("www."));
+        else
+          title = g_strdup (host);
+      }
     }
 
     soup_uri_free (uri);
@@ -1292,10 +1211,14 @@ notify_launch_cb (NotifyNotification *notification,
                   char               *action,
                   gpointer            user_data)
 {
-  char *desktop_file = user_data;
+  g_autofree char *desktop_file = user_data;
 
-  ephy_file_launch_desktop_file (desktop_file, NULL, 0, NULL);
-  g_free (desktop_file);
+  /* We can't get here under flatpak because all web app functionality
+   * is disabled when running under flatpak.
+   */
+  ephy_file_launch_desktop_file (desktop_file,
+                                 gtk_get_current_event_time (),
+                                 EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
 }
 
 static gboolean
@@ -1327,30 +1250,70 @@ confirm_web_application_overwrite (GtkWindow *parent, const char *title)
   return response == GTK_RESPONSE_OK;
 }
 
+static void
+session_bus_ready_cb (GObject      *source,
+                      GAsyncResult *res,
+                      gpointer      user_data)
+{
+  g_autoptr(GDBusConnection) connection = g_bus_get_finish (res, NULL);
+  g_autofree gchar *desktop_file = user_data;
+  g_autofree gchar *app_id = NULL;
+  GVariant *app;
+
+  if (!connection)
+    return;
+
+  app_id = g_path_get_basename (desktop_file);
+  app = g_variant_new_string (app_id);
+
+  g_dbus_connection_call (connection,
+                          "org.gnome.Shell",
+                          "/org/gnome/Shell",
+                          "org.gnome.Shell",
+                          "FocusApp",
+                          g_variant_new_tuple (&app, 1),
+                          NULL,
+                          G_DBUS_CALL_FLAGS_NO_AUTO_START,
+                          -1,
+                          NULL,
+                          NULL,
+                          NULL);
+}
+
+static void
+ephy_focus_desktop_app (const char *desktop_file)
+{
+  g_bus_get (G_BUS_TYPE_SESSION, NULL, session_bus_ready_cb, g_strdup (desktop_file));
+}
+
 static void
 dialog_save_as_application_response_cb (GtkDialog                 *dialog,
                                         gint                       response,
                                         EphyApplicationDialogData *data)
 {
-  const char *app_name;
-  char *desktop_file;
-  char *message;
-  NotifyNotification *notification;
-
   if (response == GTK_RESPONSE_OK) {
-    app_name = gtk_entry_get_text (GTK_ENTRY (data->entry));
+    const char *app_name;
+    g_autofree gchar *app_id = NULL;
+    g_autofree gchar *desktop_file = NULL;
+    char *message;
+    NotifyNotification *notification;
 
-    if (ephy_web_application_exists (app_name)) {
+    app_name = gtk_entry_get_text (GTK_ENTRY (data->entry));
+    app_id = ephy_web_application_get_app_id_from_name (app_name);
+
+    if (ephy_web_application_exists (app_id)) {
       if (confirm_web_application_overwrite (GTK_WINDOW (dialog), app_name))
-        ephy_web_application_delete (app_name);
+        ephy_web_application_delete (app_id);
       else
         return;
     }
 
     /* Create Web Application, including a new profile and .desktop file. */
-    desktop_file = ephy_web_application_create (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)),
+    desktop_file = ephy_web_application_create (app_id,
+                                                webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)),
                                                 app_name,
                                                 gtk_image_get_pixbuf (GTK_IMAGE (data->image)));
+
     if (desktop_file)
       message = g_strdup_printf (_("The application “%s” is ready to be used"),
                                  app_name);
@@ -1368,7 +1331,8 @@ dialog_save_as_application_response_cb (GtkDialog                 *dialog,
                                       g_path_get_basename (desktop_file),
                                       NULL);
       notify_notification_set_icon_from_pixbuf (notification, gtk_image_get_pixbuf (GTK_IMAGE (data->image)));
-      g_free (desktop_file);
+
+      ephy_focus_desktop_app (desktop_file);
     }
 
     notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
@@ -1456,7 +1420,7 @@ window_cmd_save_as_application (GSimpleAction *action,
   context = gtk_widget_get_style_context (label);
   gtk_style_context_add_class (context, "dim-label");
 
-  data = g_slice_new0 (EphyApplicationDialogData);
+  data = g_new0 (EphyApplicationDialogData, 1);
   data->view = view;
   data->image = image;
   data->entry = entry;
@@ -1484,25 +1448,31 @@ get_suggested_filename (EphyEmbed *embed)
   const char *mimetype;
   WebKitURIResponse *response;
   WebKitWebResource *web_resource;
+  SoupURI *soup_uri;
 
   view = ephy_embed_get_web_view (embed);
   web_resource = webkit_web_view_get_main_resource (WEBKIT_WEB_VIEW (view));
   response = webkit_web_resource_get_response (web_resource);
   mimetype = webkit_uri_response_get_mime_type (response);
+  soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
 
-  if ((g_ascii_strncasecmp (mimetype, "text/html", 9)) == 0) {
+  if (g_ascii_strncasecmp (mimetype, "text/html", 9) == 0 && g_strcmp0 (soup_uri_get_scheme (soup_uri), EPHY_VIEW_SOURCE_SCHEME) != 0) {
     /* Web Title will be used as suggested filename */
     suggested_filename = g_strconcat (ephy_embed_get_title (embed), ".mhtml", NULL);
   } else {
     suggested_filename = g_strdup (webkit_uri_response_get_suggested_filename (response));
     if (!suggested_filename) {
-      SoupURI *soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
       char *last_slash = strrchr (soup_uri->path, '/');
       suggested_filename = soup_uri_decode (last_slash ? (last_slash + 1) : soup_uri->path);
-      soup_uri_free (soup_uri);
+
+      if (!strlen (suggested_filename)) {
+        g_free (suggested_filename);
+        suggested_filename = g_strdup ("index.html");
+      }
     }
   }
 
+  soup_uri_free (soup_uri);
   return suggested_filename;
 }
 
@@ -1524,6 +1494,8 @@ save_response_cb (GtkNativeDialog *dialog, int response, EphyEmbed *embed)
       g_free (converted);
       g_free (uri);
     }
+
+    g_settings_set_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY, gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog)));
   }
 
   g_object_unref (dialog);
@@ -1548,6 +1520,7 @@ window_cmd_save_as (GSimpleAction *action,
                                      EPHY_FILE_FILTER_NONE);
 
   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), g_settings_get_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY));
 
   suggested_filename = ephy_sanitize_filename (get_suggested_filename (embed));
 
@@ -1715,7 +1688,7 @@ window_cmd_find (GSimpleAction *action,
   EphyFindToolbar *toolbar;
 
   toolbar = EPHY_FIND_TOOLBAR (ephy_window_get_current_find_toolbar (window));
-  ephy_find_toolbar_toggle_state (toolbar);
+  ephy_find_toolbar_open (toolbar, FALSE, FALSE);
 }
 
 void
@@ -1800,7 +1773,7 @@ window_cmd_zoom_normal (GSimpleAction *action,
                         gpointer       user_data)
 {
   EphyWindow *window = user_data;
-  ephy_window_set_zoom (window, 1.0);
+  ephy_window_set_zoom (window, 0.0);
 }
 
 void
@@ -1820,6 +1793,23 @@ static void
 view_source_embedded (const char *uri, EphyEmbed *embed)
 {
   EphyEmbed *new_embed;
+  SoupURI *soup_uri;
+  char *source_uri;
+
+  /* Abort if we're already in view source mode */
+  if (strstr (uri, EPHY_VIEW_SOURCE_SCHEME) == uri)
+    return;
+
+  soup_uri = soup_uri_new (uri);
+  if (!soup_uri) {
+    g_critical ("Failed to construct SoupURI for %s", uri);
+    return;
+  }
+
+  /* Convert e.g. https://gnome.org to ephy-source://gnome.org#https */
+  soup_uri_set_fragment (soup_uri, soup_uri->scheme);
+  soup_uri_set_scheme (soup_uri, EPHY_VIEW_SOURCE_SCHEME);
+  source_uri = soup_uri_to_string (soup_uri, FALSE);
 
   new_embed = ephy_shell_new_tab
                 (ephy_shell_get_default (),
@@ -1827,13 +1817,11 @@ view_source_embedded (const char *uri, EphyEmbed *embed)
                 embed,
                 EPHY_NEW_TAB_JUMP | EPHY_NEW_TAB_APPEND_AFTER);
 
-  /* FIXME: Implement embedded view source mode using a custom URI handler and a
-   * javascript library for the syntax highlighting.
-   * https://bugzilla.gnome.org/show_bug.cgi?id=731558
-   */
-  webkit_web_view_load_uri
-    (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_embed), uri);
+  webkit_web_view_load_uri (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (new_embed), source_uri);
   gtk_widget_grab_focus (GTK_WIDGET (new_embed));
+
+  g_free (source_uri);
+  soup_uri_free (soup_uri);
 }
 
 static void
@@ -1912,7 +1900,7 @@ save_temp_source_close_cb (GOutputStream *ostream, GAsyncResult *result, gpointe
     goto out;
   }
 
-  if (!ephy_file_launch_handler ("text/plain", file, gtk_get_current_event_time ())) {
+  if (!ephy_file_launch_handler (file, gtk_get_current_event_time ())) {
     /* Fallback to view the source inside the browser */
     EphyEmbed *embed;
 
@@ -2087,15 +2075,11 @@ window_cmd_page_source (GSimpleAction *action,
 
   address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 
-#if 0
- FIXME: Disabled due to bug #738475
-
   if (g_settings_get_boolean (EPHY_SETTINGS_MAIN,
                               EPHY_PREFS_INTERNAL_VIEW_SOURCE)) {
     view_source_embedded (address, embed);
     return;
   }
-#endif
 
   user_time = gtk_get_current_event_time ();
 
@@ -2103,7 +2087,7 @@ window_cmd_page_source (GSimpleAction *action,
     GFile *file;
 
     file = g_file_new_for_uri (address);
-    ephy_file_launch_handler ("text/plain", file, user_time);
+    ephy_file_launch_handler (file, user_time);
 
     g_object_unref (file);
   } else {
@@ -2259,13 +2243,16 @@ window_cmd_change_fullscreen_state (GSimpleAction *action,
                                     GVariant      *state,
                                     gpointer       user_data)
 {
+  EphyWindow *window = EPHY_WINDOW (user_data);
   gboolean active;
 
   active = g_variant_get_boolean (state);
+  ephy_window_show_fullscreen_header_bar (window);
+
   if (active)
-    gtk_window_fullscreen (GTK_WINDOW (user_data));
+    gtk_window_fullscreen (GTK_WINDOW (window));
   else
-    gtk_window_unfullscreen (GTK_WINDOW (user_data));
+    gtk_window_unfullscreen (GTK_WINDOW (window));
 
   g_simple_action_set_state (action, g_variant_new_boolean (active));
 }
@@ -2387,7 +2374,7 @@ window_cmd_tabs_detach (GSimpleAction *action,
   ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (new_window), embed, 0, FALSE);
   g_object_unref (embed);
 
-  gtk_window_present (GTK_WINDOW (new_window));
+  gtk_window_present_with_time (GTK_WINDOW (new_window), gtk_get_current_event_time ());
 }
 
 void
@@ -2413,6 +2400,142 @@ window_cmd_tabs_close (GSimpleAction *action,
   g_signal_emit_by_name (notebook, "tab-close-request", embed);
 }
 
+void
+window_cmd_tabs_close_left (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       user_data)
+{
+  EphyWindow *window = user_data;
+  GtkWidget *notebook;
+  EphyEmbed *embed;
+  int current_page_no;
+  GSList *pages_to_close = NULL;
+
+  notebook = ephy_window_get_notebook (window);
+  current_page_no = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+
+  for (int i = 0; i < current_page_no; i++) {
+    embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+    pages_to_close = g_slist_prepend (pages_to_close, embed);
+  }
+
+  for (GSList *l = pages_to_close; l != NULL; l = l->next) {
+    g_assert (l->data != NULL);
+    g_signal_emit_by_name (GTK_NOTEBOOK (notebook), "tab-close-request", l->data);
+  }
+
+  g_slist_free (pages_to_close);
+}
+
+void
+window_cmd_tabs_close_right (GSimpleAction *action,
+                             GVariant      *parameter,
+                             gpointer       user_data)
+{
+  EphyWindow *window = user_data;
+  GtkWidget *notebook;
+  EphyEmbed *embed;
+  int n_pages, current_page_no;
+  GSList *pages_to_close = NULL;
+
+  notebook = ephy_window_get_notebook (window);
+  n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+  current_page_no = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+
+  for (int i = current_page_no + 1; i < n_pages; i++) {
+    embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+    pages_to_close = g_slist_prepend (pages_to_close, embed);
+  }
+
+  for (GSList *l = pages_to_close; l != NULL; l = l->next) {
+    g_assert (l->data != NULL);
+    g_signal_emit_by_name (GTK_NOTEBOOK (notebook), "tab-close-request", l->data);
+  }
+
+  g_slist_free (pages_to_close);
+}
+
+void
+window_cmd_tabs_close_others (GSimpleAction *action,
+                              GVariant      *parameter,
+                              gpointer       user_data)
+{
+  EphyWindow *window = user_data;
+  GtkWidget *notebook;
+  EphyEmbed *embed;
+  int n_pages, current_page;
+  GSList *pages_to_close = NULL;
+
+  notebook = ephy_window_get_notebook (window);
+  n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+  current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+
+  for (int i = 0; i < n_pages; i++) {
+    if (i != current_page) {
+      embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+      pages_to_close = g_slist_prepend (pages_to_close, embed);
+    }
+  }
+
+  for (GSList *l = pages_to_close; l != NULL; l = l->next) {
+    g_assert (l->data != NULL);
+    g_signal_emit_by_name (GTK_NOTEBOOK (notebook), "tab-close-request", l->data);
+  }
+
+  g_slist_free (pages_to_close);
+}
+
+void
+window_cmd_tabs_reload (GSimpleAction *action,
+                        GVariant      *parameter,
+                        gpointer       user_data)
+{
+  EphyWindow *window = user_data;
+  EphyEmbed *embed;
+  WebKitWebView *view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+
+  view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+  webkit_web_view_reload (view);
+}
+
+void
+window_cmd_tabs_reload_all_tabs (GSimpleAction *action,
+                                 GVariant      *parameter,
+                                 gpointer       user_data)
+{
+  EphyWindow *window = user_data;
+  EphyEmbed *embed;
+  WebKitWebView *view;
+  GtkWidget *notebook;
+  int n_pages;
+
+  notebook = ephy_window_get_notebook (window);
+  n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+
+  for (int i = 0; i < n_pages; i++) {
+    embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+
+    view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+    webkit_web_view_reload (view);
+  }
+}
+
+void
+window_cmd_tabs_reopen_closed_tab (GSimpleAction *action,
+                                   GVariant      *parameter,
+                                   gpointer       user_data)
+{
+  EphySession *session = ephy_shell_get_session (ephy_shell_get_default ());
+
+  g_assert (session != NULL);
+  ephy_session_undo_close_tab (session);
+}
+
 void
 window_cmd_show_tab (GSimpleAction *action,
                      GVariant      *parameter,
@@ -2437,3 +2560,107 @@ window_cmd_change_show_tab_state (GSimpleAction *action,
 {
   /* This page intentionally left blank. */
 }
+
+void
+window_cmd_toggle_reader_mode (GSimpleAction *action,
+                               GVariant      *parameter,
+                               gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyWebView *web_view;
+  gboolean active;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  web_view = ephy_embed_get_web_view (embed);
+
+  if (!ephy_web_view_is_reader_mode_available (web_view))
+    return;
+
+  active = ephy_web_view_get_reader_mode_state (web_view);
+
+  ephy_web_view_toggle_reader_mode (web_view, !active);
+}
+
+void
+window_cmd_open_application_manager (GSimpleAction *action,
+                                     GVariant      *parameter,
+                                     gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyWebView *web_view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  web_view = ephy_embed_get_web_view (embed);
+
+  ephy_web_view_load_url (web_view, "about:applications");
+}
+
+void
+window_cmd_homepage_new_tab (GSimpleAction *action,
+                             GVariant      *parameter,
+                             gpointer       user_data)
+{
+  EphyWindow *window = EPHY_WINDOW (user_data);
+  EphyEmbed *embed;
+  EphyWebView *web_view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                              EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
+                              NULL,
+                              0);
+
+  web_view = ephy_embed_get_web_view (embed);
+  ephy_web_view_load_homepage (web_view);
+
+  ephy_embed_container_set_active_child (EPHY_EMBED_CONTAINER (window), embed);
+
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+}
+
+static void
+clipboard_text_received_cb (GtkClipboard *clipboard,
+                            const gchar  *text,
+                            EphyWindow   *window)
+{
+  EphyEmbed *embed;
+  EphyWebView *web_view;
+
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+  g_assert (embed != NULL);
+
+  embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                              EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
+                              NULL,
+                              0);
+
+  web_view = ephy_embed_get_web_view (embed);
+  ephy_web_view_load_url (web_view, text);
+
+  ephy_embed_container_set_active_child (EPHY_EMBED_CONTAINER (window), embed);
+  gtk_widget_grab_focus (GTK_WIDGET (embed));
+
+  g_object_unref (window);
+}
+
+void
+window_cmd_new_tab_from_clipboard (GSimpleAction *action,
+                                   GVariant      *parameter,
+                                   gpointer       user_data)
+{
+  EphyWindow *ephy_window = EPHY_WINDOW (user_data);
+  GtkClipboard *clipboard;
+
+  clipboard = gtk_clipboard_get_default (gdk_display_get_default ());
+  gtk_clipboard_request_text (clipboard,
+                              (GtkClipboardTextReceivedFunc)clipboard_text_received_cb,
+                              g_object_ref (ephy_window));
+}
