@@ -21,6 +21,7 @@
 #include "config.h"
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-settings.h"
 
 #include <glib.h>
@@ -64,31 +65,31 @@ test_ephy_file_helpers_init (void)
                     keep_dir ? "TRUE" : "FALSE",
                     ensure_exists ? "TRUE" : "FALSE");
 
-    g_assert (ephy_dot_dir () == NULL);
-    g_assert (ephy_file_helpers_init (NULL, test.flags, NULL));
+    g_assert_null (ephy_profile_dir ());
+    g_assert_true (ephy_file_helpers_init (NULL, test.flags, NULL));
 
     tmp_dir = g_strdup (ephy_file_tmp_dir ());
-    dot_dir = g_strdup (ephy_dot_dir ());
+    dot_dir = g_strdup (ephy_profile_dir ());
 
-    g_assert (tmp_dir != NULL);
-    g_assert (dot_dir != NULL);
+    g_assert_nonnull (tmp_dir);
+    g_assert_nonnull (dot_dir);
 
     /* Should always exist after ephy_file_tmp_dir(). */
-    g_assert (g_file_test (tmp_dir, G_FILE_TEST_EXISTS));
-    g_assert (g_file_test (dot_dir, G_FILE_TEST_EXISTS) == ensure_exists);
+    g_assert_true (g_file_test (tmp_dir, G_FILE_TEST_EXISTS));
+    g_assert_true (g_file_test (dot_dir, G_FILE_TEST_EXISTS) == ensure_exists);
 
     ephy_file_helpers_shutdown ();
 
     /* Private profiles have their dot_dir inside tmp_dir. */
-    g_assert (g_file_test (tmp_dir, G_FILE_TEST_EXISTS) == keep_dir);
-    g_assert (g_file_test (dot_dir, G_FILE_TEST_EXISTS) == (keep_dir && ensure_exists));
+    g_assert_true (g_file_test (tmp_dir, G_FILE_TEST_EXISTS) == keep_dir);
+    g_assert_true (g_file_test (dot_dir, G_FILE_TEST_EXISTS) == (keep_dir && ensure_exists));
 
     /* Cleanup dir left behind. */
     if (keep_dir) {
       /* As a safety measure, only try recursive delete on paths
        * prefixed with /tmp. */
       if (g_str_has_prefix (tmp_dir, "/tmp"))
-        g_assert (ephy_file_delete_dir_recursively (tmp_dir, NULL));
+        g_assert_true (ephy_file_delete_dir_recursively (tmp_dir, NULL));
       else
         g_warning ("INIT: dangerous path returned as tmp_dir: %s", tmp_dir);
     }
@@ -102,22 +103,24 @@ typedef struct {
   const char *key_value;
   const char *expected;
   GUserDirectory user_dir;
+  gboolean in_flatpak;
 } DownloadsDirTest;
 
 static const DownloadsDirTest downloads_tests[] =
 {
-  { "Desktop", NULL, G_USER_DIRECTORY_DESKTOP },
+  { "Desktop", NULL, G_USER_DIRECTORY_DESKTOP, FALSE },
 
-  { "Downloads", NULL, G_USER_DIRECTORY_DOWNLOAD },
-  { "invalid-keyword", NULL, G_USER_DIRECTORY_DOWNLOAD },
+  { "Downloads", NULL, G_USER_DIRECTORY_DOWNLOAD, TRUE },
+  { "invalid-keyword", NULL, G_USER_DIRECTORY_DOWNLOAD, TRUE },
 
-  { "/tmp/Downloads", "/tmp/Downloads", -1 },
+  { "/tmp/Downloads", "/tmp/Downloads", -1, FALSE },
 };
 
 static void
 test_ephy_file_get_downloads_dir (void)
 {
   guint i;
+  gboolean in_flatpak = ephy_is_running_inside_flatpak ();
 
   ephy_file_helpers_init (NULL, EPHY_FILE_HELPERS_PRIVATE_PROFILE, NULL);
 
@@ -128,6 +131,9 @@ test_ephy_file_get_downloads_dir (void)
 
     test = downloads_tests[i];
 
+    if (in_flatpak && !test.in_flatpak)
+      continue;
+
     if (test.expected != NULL)
       expected_dir = test.expected;
     else
@@ -159,7 +165,6 @@ typedef struct {
 
 static const DirTest dir_tests[] =
 {
-  { "/tmp", TRUE, FALSE, FALSE },
   { "/tmp/ephy-test-dir", FALSE, TRUE, TRUE },
   /* Unreadable */
   { "/root/ephy-test-dir", FALSE, FALSE, FALSE },
@@ -182,16 +187,16 @@ test_ephy_file_create_delete_dir (void)
 
     g_test_message ("DIR: testing for %s", test.dir);
 
-    g_assert (g_file_test (test.dir, G_FILE_TEST_EXISTS) == test.exists);
-    g_assert (ephy_ensure_dir_exists (test.dir, NULL) == (test.exists || test.can_create));
-    g_assert (g_file_test (test.dir, G_FILE_TEST_EXISTS) == (test.exists || test.can_create));
+    g_assert_true (g_file_test (test.dir, G_FILE_TEST_EXISTS) == test.exists);
+    g_assert_true (ephy_ensure_dir_exists (test.dir, NULL) == (test.exists || test.can_create));
+    g_assert_true (g_file_test (test.dir, G_FILE_TEST_EXISTS) == (test.exists || test.can_create));
 
-    g_assert (ephy_file_delete_dir_recursively (test.dir, &error) == test.can_delete);
+    g_assert_true (ephy_file_delete_dir_recursively (test.dir, &error) == test.can_delete);
     if (error)
       g_error_free (error);
 
     if (test.exists)
-      g_assert (g_file_test (test.dir, G_FILE_TEST_EXISTS) != test.can_delete);
+      g_assert_true (g_file_test (test.dir, G_FILE_TEST_EXISTS) != test.can_delete);
   }
 
   ephy_file_helpers_shutdown ();
@@ -214,6 +219,17 @@ test_ephy_file_desktop_dir (void)
   ephy_file_helpers_shutdown ();
 }
 
+static void
+delete_uri (const char *uri)
+{
+  g_autoptr(GFile) file = NULL;
+
+  g_assert_nonnull (uri);
+
+  file = g_file_new_for_uri (uri);
+  g_assert_true (g_file_delete (file, NULL, NULL));
+}
+
 static void
 test_ephy_file_create_delete_tmp (void)
 {
@@ -228,8 +244,8 @@ test_ephy_file_create_delete_tmp (void)
   tmp_file = ephy_file_tmp_filename ("test-ephy", NULL);
   tmp_path = g_build_filename (ephy_file_tmp_dir (), "test-ephy", NULL);
 
-  g_assert (tmp_file == NULL);
-  g_assert (g_file_test (tmp_path, G_FILE_TEST_EXISTS) == FALSE);
+  g_assert_null (tmp_file);
+  g_assert_false (g_file_test (tmp_path, G_FILE_TEST_EXISTS));
 
   g_free (tmp_path);
   g_free (tmp_file);
@@ -239,14 +255,14 @@ test_ephy_file_create_delete_tmp (void)
   tmp_path = g_build_filename (ephy_file_tmp_dir (), tmp_file, NULL);
 
   g_test_message ("TMP: %s", tmp_path);
-  g_assert (g_file_test (tmp_path, G_FILE_TEST_EXISTS) == FALSE);
-  g_assert (g_file_set_contents (tmp_path, "test", -1, NULL));
-  g_assert (g_file_test (tmp_path, G_FILE_TEST_EXISTS));
+  g_assert_false (g_file_test (tmp_path, G_FILE_TEST_EXISTS));
+  g_assert_true (g_file_set_contents (tmp_path, "test", -1, NULL));
+  g_assert_true (g_file_test (tmp_path, G_FILE_TEST_EXISTS));
 
   tmp_uri = g_filename_to_uri (tmp_path, NULL, NULL);
-  ephy_file_delete_uri (tmp_uri);
+  delete_uri (tmp_uri);
 
-  g_assert (g_file_test (tmp_path, G_FILE_TEST_EXISTS) == FALSE);
+  g_assert_false (g_file_test (tmp_path, G_FILE_TEST_EXISTS));
 
   g_free (tmp_uri);
   g_free (tmp_path);
@@ -256,20 +272,20 @@ test_ephy_file_create_delete_tmp (void)
   tmp_file = ephy_file_tmp_filename ("test-ephy-XXXXXX", "test-ext");
   tmp_path = g_build_filename (ephy_file_tmp_dir (), tmp_file, NULL);
 
-  g_assert (g_file_test (tmp_path, G_FILE_TEST_EXISTS) == FALSE);
-  g_assert (g_file_set_contents (tmp_path, "test", -1, NULL));
-  g_assert (g_file_test (tmp_path, G_FILE_TEST_EXISTS));
+  g_assert_false (g_file_test (tmp_path, G_FILE_TEST_EXISTS));
+  g_assert_true (g_file_set_contents (tmp_path, "test", -1, NULL));
+  g_assert_true (g_file_test (tmp_path, G_FILE_TEST_EXISTS));
 
   tmp_path_prefix = g_build_filename (ephy_file_tmp_dir (), "test-ephy-", NULL);
 
   g_test_message ("TMP: %s", tmp_path);
-  g_assert (g_str_has_suffix (tmp_path, "test-ext"));
-  g_assert (g_str_has_prefix (tmp_path, tmp_path_prefix));
+  g_assert_true (g_str_has_suffix (tmp_path, "test-ext"));
+  g_assert_true (g_str_has_prefix (tmp_path, tmp_path_prefix));
 
   tmp_uri = g_filename_to_uri (tmp_path, NULL, NULL);
-  ephy_file_delete_uri (tmp_uri);
+  delete_uri (tmp_uri);
 
-  g_assert (g_file_test (tmp_file, G_FILE_TEST_EXISTS) == FALSE);
+  g_assert_false (g_file_test (tmp_file, G_FILE_TEST_EXISTS));
 
   g_free (tmp_uri);
   g_free (tmp_path_prefix);
@@ -335,8 +351,9 @@ main (int argc, char *argv[])
   g_test_add_func ("/lib/ephy-file-helpers/get_downloads_dir",
                    test_ephy_file_get_downloads_dir);
 
-  g_test_add_func ("/lib/ephy-file-helpers/create_delete_dir",
-                   test_ephy_file_create_delete_dir);
+  if (!ephy_is_running_inside_flatpak ())
+    g_test_add_func ("/lib/ephy-file-helpers/create_delete_dir",
+                     test_ephy_file_create_delete_dir);
 
   g_test_add_func ("/lib/ephy-file-helpers/desktop_dir",
                    test_ephy_file_desktop_dir);
