@@ -19,11 +19,22 @@
  */
 
 #include "config.h"
+#include "ephy-debug.h"
+#include "ephy-file-helpers.h"
 #include "ephy-history-service.h"
 
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 
+static const char *
+test_db_filename (void)
+{
+  static char *filename = NULL;
+  if (!filename)
+    filename = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
+  return filename;
+}
+
 static EphyHistoryService *
 ensure_empty_history (const char *filename)
 {
@@ -36,10 +47,8 @@ ensure_empty_history (const char *filename)
 static void
 test_create_history_service (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
 
-  g_free (temporary_file);
   g_object_unref (service);
 }
 
@@ -47,7 +56,6 @@ static gboolean
 destroy_history_service_and_end_main_loop (EphyHistoryService *service)
 {
   g_object_unref (service);
-  g_assert (TRUE);
   gtk_main_quit ();
 
   return FALSE;
@@ -56,9 +64,7 @@ destroy_history_service_and_end_main_loop (EphyHistoryService *service)
 static void
 test_create_history_service_and_destroy_later (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
-  g_free (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
   g_timeout_add (100, (GSourceFunc)destroy_history_service_and_end_main_loop, service);
 
   gtk_main ();
@@ -68,25 +74,23 @@ static void
 page_vist_created (EphyHistoryService *service, gboolean success, gpointer result_data, gpointer user_data)
 {
   if (user_data != NULL) {
-    g_assert (EPHY_IS_HISTORY_SERVICE (user_data));
+    g_assert_true (EPHY_IS_HISTORY_SERVICE (user_data));
     g_object_unref (user_data);
   }
   g_object_unref (service);
-  g_assert (result_data == NULL);
-  g_assert (success);
+  g_assert_null (result_data);
+  g_assert_true (success);
   gtk_main_quit ();
 }
 
 static void
 test_create_history_entry (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
 
   EphyHistoryPageVisit *visit = ephy_history_page_visit_new ("http://www.gnome.org", 0, EPHY_PAGE_VISIT_TYPED);
   ephy_history_service_add_visit (service, visit, NULL, page_vist_created, NULL);
   ephy_history_page_visit_free (visit);
-  g_free (temporary_file);
 
   gtk_main ();
 }
@@ -94,16 +98,14 @@ test_create_history_entry (void)
 static void
 test_readonly_mode (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
-  EphyHistoryService *readonly_service = ephy_history_service_new (temporary_file, EPHY_SQLITE_CONNECTION_MODE_READ_ONLY);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
+  EphyHistoryService *readonly_service = ephy_history_service_new (test_db_filename (), EPHY_SQLITE_CONNECTION_MODE_READ_ONLY);
 
   /* Having the database open read-only should not break normal connections.
    * https://bugzilla.gnome.org/show_bug.cgi?id=778649 */
   EphyHistoryPageVisit *visit = ephy_history_page_visit_new ("http://www.gnome.org", 0, EPHY_PAGE_VISIT_TYPED);
   ephy_history_service_add_visit (service, visit, NULL, page_vist_created, readonly_service);
   ephy_history_page_visit_free (visit);
-  g_free (temporary_file);
 
   gtk_main ();
 }
@@ -130,15 +132,15 @@ verify_create_history_entry_cb (EphyHistoryService *service, gboolean success, g
   GList *current = visits;
   GList *current_baseline = baseline_visits;
 
-  g_assert (user_data == NULL);
-  g_assert (success);
-  g_assert (visits != NULL);
+  g_assert_null (user_data);
+  g_assert_true (success);
+  g_assert_nonnull (visits);
   g_assert_cmpint (g_list_length (visits), ==, g_list_length (baseline_visits));
 
   while (current_baseline) {
     EphyHistoryPageVisit *visit, *baseline_visit;
 
-    g_assert (current);
+    g_assert_nonnull (current);
     visit = (EphyHistoryPageVisit *)current->data;
     baseline_visit = (EphyHistoryPageVisit *)current_baseline->data;
 
@@ -161,24 +163,22 @@ verify_create_history_entry_cb (EphyHistoryService *service, gboolean success, g
 static void
 verify_create_history_entry (EphyHistoryService *service, gboolean success, gpointer result_data, gpointer user_data)
 {
-  g_assert (result_data == NULL);
+  g_assert_null (result_data);
   g_assert_cmpint (42, ==, GPOINTER_TO_INT (user_data));
-  g_assert (success);
+  g_assert_true (success);
   ephy_history_service_find_visits_in_time (service, 0, 8, NULL, verify_create_history_entry_cb, NULL);
 }
 
 static void
 test_create_history_entries (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
 
   GList *visits = create_test_page_visit_list ();
 
   /* We use 42 here just to verify that user_data is passed properly to the callback */
   ephy_history_service_add_visits (service, visits, NULL, verify_create_history_entry, GINT_TO_POINTER (42));
   ephy_history_page_visit_list_free (visits);
-  g_free (temporary_file);
 
   gtk_main ();
 }
@@ -188,8 +188,8 @@ get_url (EphyHistoryService *service, gboolean success, gpointer result_data, gp
 {
   EphyHistoryURL *url = (EphyHistoryURL *)result_data;
 
-  g_assert (success == TRUE);
-  g_assert (url != NULL);
+  g_assert_true (success);
+  g_assert_nonnull (url);
   g_assert_cmpstr (url->title, ==, "GNOME");
 
   ephy_history_url_free (url);
@@ -201,7 +201,7 @@ static void
 set_url_title (EphyHistoryService *service, gboolean success, gpointer result_data, gpointer user_data)
 {
   gboolean test_result = GPOINTER_TO_INT (user_data);
-  g_assert (success == TRUE);
+  g_assert_true (success);
 
   if (test_result == FALSE) {
     g_object_unref (service);
@@ -219,13 +219,11 @@ set_url_title_visit_created (EphyHistoryService *service, gboolean success, gpoi
 static void
 test_set_url_title_helper (gboolean test_results)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
 
   EphyHistoryPageVisit *visit = ephy_history_page_visit_new ("http://www.gnome.org", 0, EPHY_PAGE_VISIT_TYPED);
   ephy_history_service_add_visit (service, visit, NULL, set_url_title_visit_created, GINT_TO_POINTER (test_results));
   ephy_history_page_visit_free (visit);
-  g_free (temporary_file);
 
   gtk_main ();
 }
@@ -245,7 +243,7 @@ test_set_url_title_is_correct (void)
 static void
 set_url_title_url_not_existent (EphyHistoryService *service, gboolean success, gpointer result_data, gpointer user_data)
 {
-  g_assert (success == FALSE);
+  g_assert_false (success);
   g_object_unref (service);
   gtk_main_quit ();
 }
@@ -253,9 +251,7 @@ set_url_title_url_not_existent (EphyHistoryService *service, gboolean success, g
 static void
 test_set_url_title_url_not_existent (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
-  g_free (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
 
   ephy_history_service_set_url_title (service, "http://www.gnome.org", "GNOME", NULL, set_url_title_url_not_existent, NULL);
 
@@ -270,15 +266,15 @@ test_get_url_done (EphyHistoryService *service, gboolean success, gpointer resul
 
   url = (EphyHistoryURL *)result_data;
 
-  g_assert (success == expected_success);
+  g_assert_true (success == expected_success);
 
   if (expected_success == TRUE) {
-    g_assert (url != NULL);
+    g_assert_nonnull (url);
     g_assert_cmpstr (url->url, ==, "http://www.gnome.org");
     g_assert_cmpint (url->id, !=, -1);
     ephy_history_url_free (url);
   } else
-    g_assert (url == NULL);
+    g_assert_null (url);
 
   g_object_unref (service);
   gtk_main_quit ();
@@ -287,7 +283,7 @@ test_get_url_done (EphyHistoryService *service, gboolean success, gpointer resul
 static void
 test_get_url_visit_added (EphyHistoryService *service, gboolean success, gpointer result_data, gpointer user_data)
 {
-  g_assert (success == TRUE);
+  g_assert_true (success);
 
   ephy_history_service_get_url (service, "http://www.gnome.org", NULL, test_get_url_done, user_data);
 }
@@ -295,9 +291,7 @@ test_get_url_visit_added (EphyHistoryService *service, gboolean success, gpointe
 static void
 test_get_url_helper (gboolean add_entry)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
-  g_free (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
 
   if (add_entry == TRUE) {
     EphyHistoryPageVisit *visit = ephy_history_page_visit_new ("http://www.gnome.org", 0, EPHY_PAGE_VISIT_TYPED);
@@ -373,7 +367,7 @@ perform_complex_url_query (EphyHistoryService *service,
   EphyHistoryQuery *query;
   EphyHistoryURL *url;
 
-  g_assert (success == TRUE);
+  g_assert_true (success);
 
   /* Get the most visited site that contains 'k'. */
   query = ephy_history_query_new ();
@@ -392,8 +386,7 @@ perform_complex_url_query (EphyHistoryService *service,
 static void
 test_complex_url_query (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
   GList *visits;
 
   visits = create_visits_for_complex_tests ();
@@ -412,7 +405,7 @@ perform_complex_url_query_with_time_range (EphyHistoryService *service,
   EphyHistoryQuery *query;
   EphyHistoryURL *url;
 
-  g_assert (success == TRUE);
+  g_assert_true (success);
 
   /* Get the most visited site that contains 'k' that was visited since timestamp 500. */
   query = ephy_history_query_new ();
@@ -432,8 +425,7 @@ perform_complex_url_query_with_time_range (EphyHistoryService *service,
 static void
 test_complex_url_query_with_time_range (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
   GList *visits;
 
   visits = create_visits_for_complex_tests ();
@@ -467,7 +459,7 @@ perform_query_after_clear (EphyHistoryService *service,
 {
   EphyHistoryQuery *query;
 
-  g_assert (success == TRUE);
+  g_assert_true (success);
 
   /* Get 10 random sites, the query should fail. */
   query = ephy_history_query_new ();
@@ -481,8 +473,7 @@ perform_query_after_clear (EphyHistoryService *service,
 static void
 test_clear (void)
 {
-  gchar *temporary_file = g_build_filename (g_get_tmp_dir (), "epiphany-history-test.db", NULL);
-  EphyHistoryService *service = ensure_empty_history (temporary_file);
+  EphyHistoryService *service = ensure_empty_history (test_db_filename ());
   GList *visits = create_test_page_visit_list ();
 
   ephy_history_service_add_visits (service, visits, NULL, NULL, NULL);
@@ -494,8 +485,19 @@ test_clear (void)
 int
 main (int argc, char *argv[])
 {
+  int ret;
+
   gtk_test_init (&argc, &argv);
 
+  ephy_debug_init ();
+
+  if (!ephy_file_helpers_init (NULL,
+                               EPHY_FILE_HELPERS_TESTING_MODE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
+                               NULL)) {
+    g_debug ("Something wrong happened with ephy_file_helpers_init()");
+    return -1;
+  }
+
   g_test_add_func ("/embed/history/test_create_history_service", test_create_history_service);
   g_test_add_func ("/embed/history/test_create_history_service_and_destroy_later", test_create_history_service_and_destroy_later);
   g_test_add_func ("/embed/history/test_create_history_entry", test_create_history_entry);
@@ -510,5 +512,12 @@ main (int argc, char *argv[])
   g_test_add_func ("/embed/history/test_complex_url_query_with_time_range", test_complex_url_query_with_time_range);
   g_test_add_func ("/embed/history/test_clear", test_clear);
 
-  return g_test_run ();
+  ret = g_test_run ();
+
+  ephy_file_helpers_shutdown ();
+
+  if (g_file_test (test_db_filename (), G_FILE_TEST_IS_REGULAR))
+    g_assert_cmpint (g_unlink (test_db_filename ()), ==, 0);
+
+  return ret;
 }
